# -*- coding: utf-8 -*-
import copy
import abc
from typing import Dict, Type, Set, Optional
from django.core.exceptions import ValidationError

from a3dj.patches import PatchedValidationError
from a3dj.structs.base.base_field import BaseField


class StructOptions:
    fields: Dict[str, BaseField]

    verbose_name = None
    description = None
    is_component = False
    is_abstract = False


class StructMetaClass(abc.ABCMeta):

    @abc.abstractmethod
    def _check_field(cls, field_name: str, field_instance: BaseField):
        raise NotImplementedError()

    def _get_meta(cls, meta: Optional[Type[StructOptions]]) -> StructOptions:
        if meta is None:
            return StructOptions()
        else:
            return meta()

    def _check_meta(cls):
        pass

    def _add_other_fields(cls, field_name_set: Set[str]):
        pass

    def __new__(mcs, name, bases, attrs: dict, **kwargs):
        # exclude self
        parents = [b for b in bases if isinstance(b, StructMetaClass)]
        if not parents:
            return super().__new__(mcs, name, bases, attrs, **kwargs)

        # 用于继承时，排除父类的字段
        field_name_set = set()
        # new fields
        fields = dict()
        for field_name, field_instance in attrs.items():
            field_name_set.add(field_name)

            if isinstance(field_instance, BaseField):
                if field_instance.verbose_name is None:
                    field_instance.verbose_name = field_name.replace("_", " ")

                fields[field_name] = field_instance

        for field_name in fields.keys():
            attrs.pop(field_name)

        # new class
        meta = attrs.pop("Meta", None)
        new_cls = super().__new__(mcs, name, bases, attrs, **kwargs)

        # check fields
        for field_name, field_instance in fields.items():
            new_cls._check_field(field_name, field_instance)

        # set meta
        meta = new_cls._get_meta(meta)
        meta.fields = fields
        new_cls._meta = meta

        # abstract struct ends here
        if meta.is_abstract:
            return new_cls

        # check meta
        new_cls._check_meta()

        # parent fields
        mro_structs = new_cls.mro()
        if len(mro_structs) > 3:
            parent_structs = mro_structs[1: -2]

            for parent in parent_structs:
                if not hasattr(parent, '_meta'):
                    break

                parent: Type['Struct']
                parent_fields = parent.get_meta().fields

                for name, field in parent_fields.items():
                    new_field = copy.deepcopy(field)
                    if name not in field_name_set:
                        fields[name] = new_field
                        field_name_set.add(name)

        # other fields
        new_cls._add_other_fields(field_name_set)

        # bind to class
        for field_name, field_instance in fields.items():
            field_instance.contribute_to_struct(new_cls, field_name)

        return new_cls


class Struct:
    _meta: StructOptions

    @classmethod
    def get_meta(cls):
        return cls._meta

    def __init__(self, data=None, **kwargs):
        field_list = list()
        for field_name, field_instance in self.get_meta().fields.items():
            field_list.append(field_name)
            setattr(self, field_name, None)

        if data is not None:
            is_dict = isinstance(data, dict)
            for k in field_list:
                if is_dict:
                    v = data.get(k, None)
                else:
                    v = getattr(data, k, None)

                if v is not None:
                    setattr(self, k, v)

        for k, v in kwargs.items():
            if k in field_list:
                setattr(self, k, v)

    @classmethod
    def dynamic_add_field(cls, name: str, instance: BaseField):
        cls._meta.fields[name] = instance

    def clean(self):
        # 推荐只抛出ValidationError
        pass

    def _clean_fields(self):
        for field_name, field_instance in self._meta.fields.items():
            value = getattr(self, field_name, None)

            try:
                value = field_instance.clean_for_python(value, self)
            except ValidationError as e:
                e: PatchedValidationError

                e.set_field(field_name)
                raise e

            setattr(self, field_name, value)

    def full_clean(self):
        self._clean_fields()
        self.clean()

    # 仅输出时用，所以不用clean_for_python，如有需要可手动full_clean
    def to_json_dict(self) -> dict:
        rd = dict()
        for field_name, field_instance in self._meta.fields.items():
            value = getattr(self, field_name, None)
            try:
                value = field_instance.clean_for_json(value, self)
            except ValidationError as e:
                e: PatchedValidationError

                e.set_field(field_name)
                raise e

            rd[field_name] = value

        return rd

    def __repr__(self):
        return f"<{self.__class__.__name__}: object>"

    def __str__(self):
        return f"{self.__class__.__name__}"

    @classmethod
    @abc.abstractmethod
    def to_application_open_api_dict(cls) -> dict:
        raise NotImplementedError()

    @classmethod
    def mock(cls, instance=None, root_instance=None): # noqa
        this_instance = cls()
        if root_instance is None:
            root_instance = this_instance

        for field_name, field_instance in cls._meta.fields.items():
            mock_value = field_instance.mock_manager.mock(instance=this_instance, root_instance=root_instance)
            setattr(this_instance, field_name, mock_value)

        return this_instance



# -*- coding: utf-8 -*-
import uuid

from django.core.exceptions import ValidationError
from django.db.models.fields import UUIDField as DjangoUUIDField

from a3dj.structs.base.base_field import BaseField, BaseFieldMockManager, BaseFieldOpenApiManager, JsonType, FormType
from a3dj.structs.mockers import UUIDMocker, BaseMocker


class UUIDFieldOpenApiManager(BaseFieldOpenApiManager):

    @classmethod
    def _preset_form_type(cls) -> str:
        return FormType.String

    @classmethod
    def _preset_open_api_format(cls) -> str:
        return 'uuid'

    @classmethod
    def _preset_json_type(cls) -> str:
        return JsonType.String


class UUIDFieldMockManager(BaseFieldMockManager):

    def _preset_default_mocker(self) -> BaseMocker:
        return UUIDMocker()


class UUIDField(BaseField):

    @classmethod
    def support_form(cls) -> bool:
        return True

    @classmethod
    def support_json(cls) -> bool:
        return True

    @classmethod
    def _build_from_model_field(cls, field_instance: DjangoUUIDField) -> 'UUIDField':
        return cls()

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return UUIDFieldOpenApiManager(self)

    def _preset_mock_manager(self) -> BaseFieldMockManager:
        return UUIDFieldMockManager(self)

    def to_python(self, value) -> uuid.UUID:
        if isinstance(value, uuid.UUID):
            return value

        try:
            return uuid.UUID(value)
        except (AttributeError, ValueError):
            raise ValidationError(
                DjangoUUIDField.default_error_messages["invalid"],
                code="invalid",
                params={"value": value},
            )

    def to_json(self, value: uuid.UUID) -> str:
        return str(value)



# -*- coding: utf-8 -*-
from django.http import JsonResponse

from a3dj.errors import Error


def build_success_response(data: dict = None):
    success_result = {
        'status': 'OK'
    }

    if data is not None:
        success_result['data'] = data
    return JsonResponse(data=success_result)


def build_error_response(e: Error):
    return JsonResponse(data={
        'status': e.status,
        'error_message': str(e)
    })


class FileResponseStruct:

    @classmethod
    def to_application_open_api_dict(cls):
        # 这个导入Apifox时不能正确显示，是Apifox的锅
        return {
            "application/octet-stream": {
                "schema": {
                    "type": "string",
                    "format": "binary"
                }
            }
        }



# -*- coding: utf-8 -*-
import datetime
from typing import Optional

from .base_mocker import BaseMocker


class FloatMocker(BaseMocker):

    def to_json_mock_string(self) -> Optional[str]:
        return "@float"

    def to_form_mock_string(self) -> Optional[str]:
        return "{% mock 'float' %}"

    def mock(self, _, __) -> datetime.date:
        return self.f.pyfloat()



# -*- coding: utf-8 -*-
from typing import Type, TYPE_CHECKING

from django.core.exceptions import ValidationError
from django.db.models import Field as ModelField
from django.utils.translation import gettext_lazy as _

from a3dj.core.inner_errors import ModelFieldTranslateError, MixingError
from a3dj.structs.base.base_field import BaseField, JsonType, BaseFieldMockManager, BaseFieldOpenApiManager
from a3dj.structs.mockers import ObjectMocker, BaseMocker

if TYPE_CHECKING:
    from a3dj.structs.json_structs.base_json_struct import BaseJsonStruct


class ObjectFieldOpenApiManager(BaseFieldOpenApiManager):
    field: 'ObjectField'

    @classmethod
    def _preset_form_type(cls) -> str:
        raise MixingError("FormStruct cannot use ObjectField")

    @classmethod
    def _preset_open_api_format(cls) -> str:
        return 'object'

    @classmethod
    def _preset_json_type(cls) -> str:
        return JsonType.Object

    def _build_json_base_info(self) -> dict:
        info = super()._build_json_base_info()
        obj_info = self.field.obj_cls.to_json_open_api_dict()
        info.update(obj_info)
        return info

    def get_components(self) -> dict:
        components = dict()
        sub_components = self.field.obj_cls.get_open_api_components()
        if isinstance(sub_components, dict) and len(sub_components) > 0:
            components.update(sub_components)

        return components


class ObjectFieldMockManager(BaseFieldMockManager):
    field: 'ObjectField'

    def _preset_default_mocker(self) -> BaseMocker:
        return ObjectMocker(obj_cls=self.field.obj_cls)


class ObjectField(BaseField):

    default_error_messages = {
        "invalid_type": _("This field's type must be a correct object or a dict, not “%(value)s”."),
    }

    @classmethod
    def _build_from_model_field(cls, field_instance: ModelField) -> 'BaseField':
        raise ModelFieldTranslateError("No field can translate to ObjectField")

    @classmethod
    def support_form(cls) -> bool:
        return False

    @classmethod
    def support_json(cls) -> bool:
        return True

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return ObjectFieldOpenApiManager(self)

    def _preset_mock_manager(self) -> BaseFieldMockManager:
        return ObjectFieldMockManager(self)

    def __init__(self, obj_cls: Type['BaseJsonStruct'], *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.obj_cls = obj_cls

    def to_python(self, value) -> 'BaseJsonStruct':
        if isinstance(value, dict):
            obj_instance = self.obj_cls(**value)
        elif isinstance(value, self.obj_cls):
            obj_instance = value
        else:
            raise ValidationError(
                message=self.default_error_messages["invalid_type"],
                code="invalid_type",
                params={"value": type(value)}
            )

        obj_instance.full_clean()
        return obj_instance

    def to_json(self, value: 'BaseJsonStruct') -> dict:
        return value.to_json_dict()



# -*- coding: utf-8 -*-
import copy
import json
import inspect
import os.path
from importlib import import_module
from typing import List, Dict, Type

from django.apps import AppConfig
from django.conf import settings
from django.urls import URLPattern, URLResolver
from django.utils.module_loading import import_string

from a3dj.core.inner_errors import OpenApiDocError
from a3dj.structs.json_structs.base_json_struct import BaseJsonStruct
from a3dj.errors import Error
from a3dj.view.base_post_view import BasePostView
from .view_open_api_util import ViewOpenApiUtil


def _get_cls_module_errors(cls) -> dict:
    rd = dict()

    module = import_string(cls.__module__)
    for name in dir(module):
        if name.startswith('_'):
            continue

        t = getattr(module, name)
        if inspect.isclass(t) and t is not Error and issubclass(t, Error):
            rd[t.__name__] = t
    return rd


def _get_app_name_by_urls_path(urls_path: str) -> str:
    app_path = copy.deepcopy(urls_path)

    while True:
        app_path = app_path.rsplit('.', 1)[0]
        try:
            apps_module = import_module(f'{app_path}.apps')
            for name in dir(apps_module):
                if name.startswith('_'):
                    continue

                t = getattr(apps_module, name)
                if inspect.isclass(t) and t is not AppConfig and issubclass(t, AppConfig):
                    return getattr(t, 'verbose_name', None) or getattr(t, 'name')
        except Exception as e:
            if '.' not in app_path:
                # url不在一个标准app中
                raise OpenApiDocError(f"`{urls_path}` can not find related AppConfig, {e}.")
            else:
                continue
        # app文件，但没有标准的AppConfig
        raise OpenApiDocError(f"`{urls_path}` can not find related AppConfig.")


class OpenApiDocGenerator:

    def __init__(self, project_name: str):
        self.project_name = project_name

        self._middleware_errors = dict()

        self._view_info_list = list()
        self._view2all_error_list: Dict[Type[BasePostView], List[Type[Error]]] = dict()
        self.__all_view2errors: Dict[Type[BasePostView], Dict[str, Type[Error]]] = dict()
        self._components = dict()

    def _write_doc(self, doc: dict):
        str_doc = json.dumps(doc, indent=2, ensure_ascii=False)
        filename = os.path.join(settings.BASE_DIR, f'{self.project_name}.json')
        with open(filename, 'w', encoding='utf-8') as fd:
            fd.write(str_doc)

    def _collect_errors_from_middlewares(self):
        for middleware_path in reversed(settings.MIDDLEWARE):
            middleware = import_string(middleware_path)
            errors = _get_cls_module_errors(middleware)

            self._middleware_errors.update(errors)

    def _collect_views(self):
        def _get_view_errors(vc: Type[BasePostView]):

            def __get_unit_view_errors(cls) -> dict:
                cached_errors = self.__all_view2errors.get(cls)
                if cached_errors is not None:
                    return cached_errors
                errors = _get_cls_module_errors(cls)
                self.__all_view2errors[cls] = errors
                return errors

            view_cls_errors = __get_unit_view_errors(vc)

            all_parents_errors = dict()
            for parent_cls in vc.__mro__[1:]:
                if not issubclass(parent_cls, BasePostView):
                    break
                parent_errors = __get_unit_view_errors(parent_cls)
                all_parents_errors.update(parent_errors)

            all_errors = dict()
            all_errors.update(view_cls_errors)
            all_errors.update(all_parents_errors)
            all_errors.update(self._middleware_errors)

            self._view2all_error_list[vc] = list(all_errors.values())

        def _extract_views(urlpatterns, parent_path: str = '', app_name: str = None) -> list:
            view_info_list = list()
            for p in urlpatterns:
                if isinstance(p, URLPattern):
                    callback = getattr(p, 'callback', {})
                    view_cls = getattr(callback, 'view_class', None)
                    if view_cls is not None and issubclass(view_cls, BasePostView):

                        _get_view_errors(view_cls)

                        last_path_name = str(p.pattern)
                        view_info_list.append(
                            (view_cls, parent_path + last_path_name, p.name or last_path_name, app_name)
                        )

                elif isinstance(p, URLResolver):
                    next_urlpatterns = getattr(p, 'url_patterns', None)

                    if next_urlpatterns is not None:
                        if inspect.ismodule(p.urlconf_module):
                            urls_path = p.urlconf_module.__name__
                            app_name = _get_app_name_by_urls_path(urls_path)

                        view_info_list.extend(_extract_views(next_urlpatterns, parent_path + str(p.pattern), app_name))

            return view_info_list

        urlconf = import_string(settings.ROOT_URLCONF)
        self._view_info_list = _extract_views(urlconf.urlpatterns)

    def _build_views_doc(self) -> dict:
        rd = dict()
        for view_cls, path, view_name, app_name in self._view_info_list:
            view_cls: Type[BasePostView]

            # request
            request_struct_cls = view_cls.request_struct_cls
            if request_struct_cls is None:
                request_dict = ViewOpenApiUtil.render_empty_request_application_content()
            else:
                request_dict = request_struct_cls.to_application_open_api_dict()

            # response
            response_dict = ViewOpenApiUtil.render_success_response_struct(view_cls.response_struct_cls)
            # build ordered error response
            for error in self._view2all_error_list[view_cls]:
                response_dict.update(ViewOpenApiUtil.render_error_response_struct(error))

            description = view_cls.__doc__
            if description is not None:
                description = description.strip()
            else:
                description = ''

            vd = {
                f"/{path}": {
                    "post": {
                        "summary": view_name,
                        "x-apifox-folder": app_name,
                        "deprecated": False,
                        "description": description,
                        "requestBody": {
                            "content": request_dict
                        },
                        "responses": response_dict
                    }
                }
            }
            rd.update(vd)

            if inspect.isclass(view_cls.request_struct_cls) and issubclass(view_cls.request_struct_cls, BaseJsonStruct):
                request_components = view_cls.request_struct_cls.get_open_api_components()
                self._components.update(request_components)

            if hasattr(view_cls.response_struct_cls, 'get_open_api_components'):
                response_components = view_cls.response_struct_cls.get_open_api_components()
                self._components.update(response_components)
        return rd

    def _build_components_doc(self) -> dict:
        rd = dict()
        for name, component in self._components.items():
            rd[name] = component.to_open_api_component_dict()
        return rd

    def start(self):
        self._collect_errors_from_middlewares()
        self._collect_views()

        paths = self._build_views_doc()
        components = self._build_components_doc()

        doc = {
            "openapi": "3.1.0",
            "info": {
                "title": self.project_name,
            },
            "paths": paths,
            "components": {
                'schemas': components
            }
        }

        self._write_doc(doc)



# -*- coding: utf-8 -*-
from typing import Optional

from .base_mocker import BaseMocker


class NoneMocker(BaseMocker):

    def to_json_mock_string(self) -> Optional[str]:
        return

    def to_form_mock_string(self) -> Optional[str]:
        return

    def mock(self, _, __):
        return



# -*- coding: utf-8 -*-
import tempfile
import json
from typing import List, Type, Dict

from django.http import JsonResponse, FileResponse
from django.test import TestCase as DjangoTestCase
from a3faker import FakerProxy

from a3dj.core.constant import RequestContentType
from a3dj import errors


class TestCase(DjangoTestCase):
    _custom_errors: Dict[str, Type[errors.Error]]
    custom_error_class_list: List[Type[errors.Error]] = list()

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls._custom_errors = dict()
        for error_class in cls.custom_error_class_list:
            cls._custom_errors[error_class.__name__] = error_class

    def post_json(
            self,
            path: str,
            data: dict = None,
            post_kwargs: dict = None
    ) -> dict:
        post_kwargs = post_kwargs or dict()
        post_kwargs['content_type'] = RequestContentType.Json

        return self._post(
            path=path,
            data=json.dumps(data),
            post_kwargs=post_kwargs
        )

    def post_json_return_file(
            self,
            path: str,
            data: dict = None,
            post_kwargs: dict = None,
            file_extension: str = None,
            file_kwargs: dict = None
    ) -> tempfile.NamedTemporaryFile:
        post_kwargs = post_kwargs or dict()
        post_kwargs['content_type'] = RequestContentType.Json

        return self._post(
            path=path,
            data=json.dumps(data),
            post_kwargs=post_kwargs,
            file_extension=file_extension,
            file_kwargs=file_kwargs
        )

    def post_form(
            self,
            path: str,
            data: dict = None,
            post_kwargs: dict = None
    ) -> dict:
        return self._post(
            path=path,
            data=data,
            post_kwargs=post_kwargs
        )

    def post_form_return_file(
            self,
            path: str,
            data: dict = None,
            post_kwargs: dict = None,
            file_extension: str = None,
            file_kwargs: dict = None
    ) -> tempfile.NamedTemporaryFile:
        return self._post(
            path=path,
            data=data,
            post_kwargs=post_kwargs,
            file_extension=file_extension,
            file_kwargs=file_kwargs
        )

    def _post(
            self,
            path: str,
            data=None,
            file_extension: str = None,
            post_kwargs: dict = None,
            file_kwargs: dict = None
    ):
        post_kwargs = post_kwargs or {}
        file_kwargs = file_kwargs or {}
        if file_extension is not None:
            file_kwargs['suffix'] = f'.{file_extension}'

        response = self.client.post(path, data=data, **post_kwargs)
        assert response.status_code == 200

        if isinstance(response, JsonResponse):
            rd: dict = response.json()  # noqa
            status = rd.pop('status')
            if status == 'OK':
                return rd.get('data')
            else:
                error_cls = self._custom_errors.get(status, None)
                if error_cls is None:
                    error_cls = getattr(errors, status, None)
                    assert error_cls is not None

                # 临时切一下，以方便Error子类构造函数重写
                _init = error_cls.__init__  # noqa
                error_cls.__init__ = errors.Error.__init__  # noqa
                e = error_cls(**rd)
                error_cls.__init__ = _init  # noqa
                raise e
        elif isinstance(response, FileResponse):
            tf = tempfile.NamedTemporaryFile('wb', **file_kwargs)
            for content in response.streaming_content:
                tf.write(content)
            tf.flush()

            return tf

    def setUp(self) -> None:
        super().setUp()
        self.f = FakerProxy.get_faker()



# -*- coding: utf-8 -*-
from typing import Type

from a3dj.errors import Error
from a3dj.structs import json_structs
from a3dj.structs.mockers import FixedValueMocker


class ViewOpenApiUtil:

    @classmethod
    def render_empty_request_application_content(cls) -> dict:
        return {
            "application/json": {
                "schema": {
                    "type": "object",
                    "properties": {},
                    "x-apifox-ignore-properties": [],
                    "x-apifox-orders": []
                }
            }
        }

    @classmethod
    def render_success_response_struct(cls, s: json_structs.JsonStruct) -> dict:
        return {
            "200": {
                "description": "OK",
                "content": s.to_application_open_api_dict()
            }
        }

    @classmethod
    def render_error_response_struct(cls, err: Type[Error]) -> dict:
        class ErrorJsonStruct(json_structs.JsonStruct):
            status = json_structs.CharField(mocker=FixedValueMocker(err.__name__))
            error_message = json_structs.CharField(mocker=FixedValueMocker(str(err.default_message)))

        return {
            f'x-200:{err.__name__}': {
                "description": err.__name__,
                "content": ErrorJsonStruct.to_application_open_api_dict()
            }
        }



# -*- coding: utf-8 -*-
from typing import Tuple, Any, Type
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

from a3dj.utils.hint_utils import TypeA3Model
from a3dj.structs.base.base_field import BaseField, BaseFieldMockManager, BaseFieldOpenApiManager
from .list_field import ListField
from .related_field_translater import RelatedFieldTranslater
from .model_instance_field import ModelInstanceField


class ModelMultipleInstanceField(ModelInstanceField):
    default_error_messages = {
        'not_exist': _("Model instance “%(value)s“ does not exist")
    }

    def __init__(
            self, model_cls: Type[TypeA3Model], to_field_name: str = None, query: dict = None,
            *args, **kwargs
    ):
        self.model_cls = model_cls
        self.to_field_name = to_field_name or 'id'
        self.query = query or dict()

        field_instance = getattr(model_cls, '_meta').get_field(self.to_field_name)
        item_type = RelatedFieldTranslater.translate(field_instance)
        self.outward_field: BaseField = ListField(item_type=item_type, required=False, unique=True)

        BaseField.__init__(self, *args, **kwargs)

    def to_python(self, value):
        vl = self.outward_field.to_python(value)
        fd = {f"{self.to_field_name}__in": vl}
        fd.update(self.query)

        instance_list = self.model_cls.objects.filter(**fd)
        if len(instance_list) != len(vl):
            cl = list()
            for instance in instance_list:
                cl.append(getattr(instance, self.to_field_name))

            raise ValidationError(
                message=self.default_error_messages["not_exist"],
                code="not_exist",
                params={"value": ','.join(list(set(vl) - set(cl)))}
            )

        return instance_list

    def check_has_changed(self, old_value, new_value) -> Tuple[bool, Any, Any]:
        ovl = list()
        for old in old_value or list():
            o = getattr(old, self.to_field_name)
            ovl.append(o)

        nvl = list()
        for ne in new_value or list():
            n = getattr(ne, self.to_field_name)
            nvl.append(n)

        if set(ovl) == set(nvl):
            return False, None, None
        else:
            return True, ovl, nvl

    def to_json(self, value):
        rl = list()
        for v in value.all():
            rl.append(getattr(v, self.to_field_name))
        return rl

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return self.outward_field.open_api_manager

    def _preset_mock_manager(self) -> BaseFieldMockManager:
        return self.outward_field.mock_manager

    @classmethod
    def support_json(cls) -> bool:
        return True

    @classmethod
    def support_form(cls) -> bool:
        return True



# -*- coding: utf-8 -*-
from a3dj.structs.base.struct import Struct


class BaseFormStruct(Struct):

    @classmethod
    def to_application_open_api_dict(cls) -> dict:
        return {
            'multipart/form-data': {
                "schema": cls.to_form_open_api_dict()
            }
        }

    @classmethod
    def to_form_open_api_dict(cls) -> dict:
        order_list = list()
        required_list = list()
        properties = dict()

        for field_name, field_instance in cls._meta.fields.items():
            order_list.append(field_name)

            if field_instance.required:
                required_list.append(field_name)

            properties[field_name] = field_instance.open_api_manager.to_form_open_api_dict()

        return {
            'type': 'object',
            'properties': properties,
            'required': required_list,
            'x-apifox-orders': order_list
        }



# -*- coding: utf-8 -*-
from typing import Optional

from .base_mocker import BaseMocker


class GenericIPAddressMocker(BaseMocker):

    def __init__(self, protocol: str = 'both'):
        super().__init__()
        self.protocol = protocol

    def to_json_mock_string(self) -> Optional[str]:
        return "@ip"

    def to_form_mock_string(self) -> Optional[str]:
        return "{% mock 'ip' %}"

    def mock(self, _, __) -> str:
        if self.protocol == 'ipv6':
            return self.f.ipv6()
        else:
            return self.f.ipv4()



# -*- coding: utf-8 -*-
import base64
from typing import Any, Tuple

from django.core.exceptions import ValidationError
from django.db.models import FileField as ModelFileField
from django.forms import FileField as DjangoFormFileField
from django.core.files import File
from django.core.files.temp import NamedTemporaryFile
from django.db.models.fields.files import FieldFile

from a3dj.core.validators import FileMaxSizeValidator
from a3dj.structs.base.base_field import BaseField, BaseFieldMockManager, BaseFieldOpenApiManager, JsonType, FormType
from a3dj.structs.mockers import NoneMocker, BaseMocker


class FileFieldOpenApiManger(BaseFieldOpenApiManager):
    field: 'FileField'

    @classmethod
    def _preset_form_type(cls) -> str:
        return FormType.File

    @classmethod
    def _preset_open_api_format(cls) -> str:
        # 通过Json传来的，只能是base64
        return 'base64'

    @classmethod
    def _preset_json_type(cls) -> str:
        return JsonType.String


class FileFieldMockManager(BaseFieldMockManager):
    field: 'FileField'

    def _preset_default_mocker(self) -> BaseMocker:
        return NoneMocker()


class FileField(BaseField):

    @classmethod
    def support_form(cls) -> bool:
        return True

    @classmethod
    def support_json(cls) -> bool:
        return True

    @classmethod
    def _build_from_model_field(cls, field_instance: ModelFileField) -> 'FileField':
        return cls(max_length=field_instance.max_length)

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return FileFieldOpenApiManger(self)

    def _preset_mock_manager(self) -> BaseFieldMockManager:
        return FileFieldMockManager(self)

    def __init__(self, max_length: int = None, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.max_length = max_length
        if self.max_length is not None:
            self.validators.append(FileMaxSizeValidator(self.max_length))

    def to_python(self, value) -> File:
        if isinstance(value, File):
            return value

        if isinstance(value, str):
            body = base64.b64decode(value.encode("ascii"))

            temp_file = NamedTemporaryFile()
            temp_file.write(body)
            temp_file.flush()

            return File(file=temp_file)

        raise ValidationError(DjangoFormFileField.default_error_messages["invalid"], code="invalid")

    def to_json(self, value: FieldFile) -> str:
        return value.url

    def check_has_changed(self, old_value: FieldFile, new_value: FieldFile) -> Tuple[bool, Any, Any]:
        return True, str(old_value), str(new_value)



# -*- coding: utf-8 -*-
from a3dj.structs.base.base_field import BaseField
from a3dj.structs.base.struct import StructMetaClass, StructOptions
from a3dj.structs.fields import ListField, ObjectField
from a3dj.core.inner_errors import FieldError

from .base_form_struct import BaseFormStruct


FormStructOptions = StructOptions


class FormStructMetaClass(StructMetaClass):

    def _check_field(cls, field_name, field_instance: BaseField):
        if not field_instance.support_form():
            raise FieldError(
                f"field: {field_name}, class: {field_instance.__class__.__name__}, not support form struct."
            )
        if isinstance(field_instance, ListField) and \
                isinstance(field_instance.item_type, (ListField, ObjectField)):
            raise FieldError(
                f"list field: {field_name}, not support item class: {field_instance.item_type.__class__.__name__}"
            )


class FormStruct(BaseFormStruct, metaclass=FormStructMetaClass):
    pass



# -*- coding: utf-8 -*-
from django.core.exceptions import ValidationError, NON_FIELD_ERRORS
from typing import Union


__raw_str__ = ValidationError.__str__


class _PatchedValidationError:
    error_list: list
    index_list: list
    message: str
    params: dict

    def _check_has_patched(self) -> bool:
        return getattr(self, '_has_patched', False)

    def _ensure_instance_member(self):
        has_patched = self._check_has_patched()
        if not has_patched:
            self._has_patched = True
            self.index_list = list()
            self.field_name_list = list()

    def set_field(self, field_name: str):
        self._ensure_instance_member()

        if len(self.index_list) > 0:
            for index in self.index_list:
                field_name += f'[{index}]'
            self.index_list = list()

        self.field_name_list.insert(0, field_name)

    def set_index(self, index: int):
        self._ensure_instance_member()
        self.index_list.insert(0, index)

    def _patched_str(self) -> str:
        if len(self.field_name_list) == 0:
            field_name = NON_FIELD_ERRORS
        else:
            field_name = '.'.join(self.field_name_list)
        error_message = self.message
        if self.params:
            error_message %= self.params
        return f'{field_name}: {error_message}'

    def __str__(self):
        if self._check_has_patched():
            return self._patched_str()
        else:
            if hasattr(self, "error_dict"):
                field = next(iter(self.error_dict))
                error_message = str(self.error_dict[field][0])
                return f'{field}: {error_message}'
            else:
                error = self.error_list[0]
                error_message = error.message
                if error.params:
                    error_message %= error.params
                return error_message


def _patch_validation_error():
    ValidationError._check_has_patched = _PatchedValidationError._check_has_patched # noqa
    ValidationError._ensure_instance_member = _PatchedValidationError._ensure_instance_member # noqa
    ValidationError.set_field = _PatchedValidationError.set_field
    ValidationError.set_index = _PatchedValidationError.set_index
    ValidationError._patched_str = _PatchedValidationError._patched_str # noqa
    ValidationError.__str__ = _PatchedValidationError.__str__


PatchedValidationError = Union[ValidationError, _PatchedValidationError]



# -*- coding: utf-8 -*-
from django.core import validators
from django.db.models.fields import GenericIPAddressField as DjangoGenericIPAddressField
from django.utils.ipv6 import clean_ipv6_address

from a3dj.structs.base.base_field import BaseFieldMockManager, BaseFieldOpenApiManager, JsonType, FormType
from a3dj.structs.mockers import GenericIPAddressMocker, BaseMocker
from .char_field import CharField


class GenericIPAddressFieldOpenApiManager(BaseFieldOpenApiManager):

    @classmethod
    def _preset_form_type(cls) -> str:
        return FormType.String

    @classmethod
    def _preset_open_api_format(cls) -> str:
        return 'ip'

    @classmethod
    def _preset_json_type(cls) -> str:
        return JsonType.String


class GenericIPAddressFieldMockManager(BaseFieldMockManager):

    def _preset_default_mocker(self) -> BaseMocker:
        return GenericIPAddressMocker()


class GenericIPAddressField(CharField):

    def __init__(self, protocol="both", unpack_ipv4=False, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.protocol = protocol
        self.unpack_ipv4 = unpack_ipv4
        self.validators.extend(validators.ip_address_validators(protocol, unpack_ipv4)[0])

    @classmethod
    def _build_from_model_field(cls, field_instance: DjangoGenericIPAddressField) -> 'GenericIPAddressField':
        return cls(
            protocol=field_instance.protocol, unpack_ipv4=field_instance.unpack_ipv4,
            max_length=field_instance.max_length, choices=field_instance.choices
        )

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return GenericIPAddressFieldOpenApiManager(self)

    def _preset_mock_manager(self) -> BaseFieldMockManager:
        return GenericIPAddressFieldMockManager(self)

    def to_python(self, value) -> str:
        value = super().to_python(value).strip()
        if value and ":" in value:
            return clean_ipv6_address(value, self.unpack_ipv4)
        return value



# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand
from django.conf import settings

from a3dj.core.open_api_utils import OpenApiDocGenerator


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('--project-name', '-n', dest='project_name', required=False, type=str)

    def handle(self, *args, **options):
        project_name = options.pop('project_name')
        if project_name is None:
            project_name = settings.BASE_DIR.name

        OpenApiDocGenerator(project_name=project_name).start()



# -*- coding: utf-8 -*-
from django.utils.translation import gettext_lazy as _

from .error import Error


class ServerError(Error):
    default_message = _('Server error.')


class SUnknownError(ServerError):
    default_message = _('This is a unknown logic error, please contact customer service.')


class SKnownError(ServerError):
    default_message = _('It is a known server error, please contact customer service.')



# -*- coding: utf-8 -*-
from typing import TypeVar

from django.db import models


class A3Model(models.Model):
    objects: models.QuerySet
    id: int

    class Meta:
        abstract = True


TypeA3Model = TypeVar('TypeA3Model', bound=A3Model)



# -*- coding: utf-8 -*-
from typing import Iterable, List

from django.core.exceptions import ValidationError
from django.core.validators import MinLengthValidator, MaxLengthValidator
from django.db.models import ManyToManyField as ModelManyToManyField
from django.utils.translation import gettext_lazy as _

from a3dj.structs.base.base_field import BaseField, BaseFieldMockManager, BaseFieldOpenApiManager, JsonType, FormType
from a3dj.structs.mockers import ListMocker, BaseMocker
from a3dj.patches import PatchedValidationError
from a3dj.core.utils import set_meaning_kv
from a3dj.core.inner_errors import ModelFieldTranslateError


class ListFieldOpenApiManager(BaseFieldOpenApiManager):
    field: 'ListField'

    @classmethod
    def _preset_form_type(cls) -> str:
        return FormType.Array

    @classmethod
    def _preset_open_api_format(cls) -> str:
        return 'array'

    @classmethod
    def _preset_json_type(cls) -> str:
        return JsonType.Array

    def _build_json_base_info(self) -> dict:
        info = super()._build_json_base_info()
        info['items'] = self.field.item_type.open_api_manager.to_json_open_api_dict()

        set_meaning_kv(info, 'minItems', self.field.min_length)
        set_meaning_kv(info, 'maxItems', self.field.max_length)
        set_meaning_kv(info, 'uniqueItems', self.field.unique)
        return info

    def get_components(self) -> dict:
        components = dict()
        sub_components = self.field.item_type.open_api_manager.get_components()
        if isinstance(sub_components, dict) and len(sub_components) > 0:
            components.update(sub_components)

        return components


class ListFieldMockManager(BaseFieldMockManager):
    field: 'ListField'

    def _preset_default_mocker(self) -> BaseMocker:
        return ListMocker(self.field.item_type, self.field.min_length, self.field.max_length, self.field.unique)


class ListField(BaseField):

    @classmethod
    def support_form(cls) -> bool:
        return True

    @classmethod
    def support_json(cls) -> bool:
        return True

    @classmethod
    def _build_from_model_field(cls, field_instance: ModelManyToManyField) -> 'BaseField':
        raise ModelFieldTranslateError("No field can translate to ListField")

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return ListFieldOpenApiManager(self)

    def _preset_mock_manager(self) -> BaseFieldMockManager:
        return ListFieldMockManager(self)

    default_error_messages = {
        "unique": _("“%(value)s“ already exists."),
        "invalid_type": _("This field's type must be a correct list, not “%(value)s”."),
    }

    def __init__(
            self, item_type: BaseField, min_length: int = None, max_length: int = None, unique: bool = False,
            *args, **kwargs
    ):
        super().__init__(*args, **kwargs)
        self.item_type = item_type

        self.min_length = min_length
        self.max_length = max_length
        self.unique = unique

        if self.min_length is not None:
            self.validators.append(MinLengthValidator(self.min_length))

        if self.max_length is not None:
            self.validators.append(MaxLengthValidator(self.max_length))

    def validate(self, value, struct_instance):
        value = super().validate(value, struct_instance)
        if self.unique and isinstance(value, Iterable):
            seen = list()
            for i, item in enumerate(value):
                if item in seen:
                    e: PatchedValidationError = ValidationError(
                        message=self.default_error_messages["unique"],
                        code="unique",
                        params={"value": str(item)},
                    )
                    e.set_index(i)
                    raise e

                seen.append(item)

        return value

    def to_python(self, value: List) -> List:
        if not isinstance(value, list):
            raise ValidationError(
                message=self.default_error_messages["invalid_type"],
                code="invalid_type",
                params={"value": type(value)}
            )

        rl = list()
        for i, item in enumerate(value):
            try:
                v = self.item_type.clean_for_python(item, None)
            except ValidationError as e:
                e: PatchedValidationError

                e.set_index(i)
                raise e
            rl.append(v)

        return rl

    def to_json(self, value: List) -> List:
        rl = list()
        for item in value:
            v = self.item_type.to_json(item)
            rl.append(v)
        return rl



# -*- coding: utf-8 -*-


class Error(Exception):
    default_message = 'error'

    def __init__(self, error_message: str = None, detail_message: str = None):
        self.status = self.__class__.__name__
        self.error_message = error_message or self.default_message
        self.default_message = detail_message or ''

    def __str__(self):
        return f"{self.error_message}"

    def __repr__(self):
        return f"{self.default_message}"



# -*- coding: utf-8 -*-
import traceback
from django.http import HttpRequest

from a3py.improved.json import fast_loads
from .constant import RequestContentType


def get_ip(request: HttpRequest) -> str:
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip


def get_user_agent(request: HttpRequest) -> str:
    return request.META.get('HTTP_USER_AGENT', '')[:256]


def get_referer(request: HttpRequest) -> str:
    return request.META.get('HTTP_REFERER')


def get_full_url(request: HttpRequest) -> str:
    return request.build_absolute_uri()


def get_url_path(request: HttpRequest) -> str:
    return request.path


def get_host(request: HttpRequest) -> str:
    return '{}://{}'.format(request.scheme, request.META['HTTP_HOST'])


def get_form_data(request: HttpRequest) -> dict:
    return dict(request.POST.lists())


def get_json_data(request: HttpRequest) -> dict:
    return fast_loads(request.body)


def get_request_data(request: HttpRequest) -> dict:
    if request.content_type == RequestContentType.FormData:
        return get_form_data(request)
    elif request.content_type == RequestContentType.Json:
        return get_json_data(request)


def build_critical_message(request: HttpRequest, err: Exception) -> str:
    return f"\n" \
           f"ip: {get_ip(request)}\n" + \
           f"url: {get_full_url(request)}\n" + \
           f"content-type: {request.content_type}\n" + \
           f"request: {get_request_data(request)}\n" + \
           f"str: {str(err)}\n" + \
           f"repr: {repr(err)}\n" + \
           f"traceback: {traceback.format_exc()}\n"



# -*- coding: utf-8 -*-
from a3dj.structs.base.base_field import BaseFieldOpenApiManager
from .integer_field import IntegerField, IntegerFieldOpenApiManager


class PositiveIntegerFieldOpenApiManager(IntegerFieldOpenApiManager):
    field: 'PositiveIntegerField'

    @classmethod
    def _preset_open_api_format(cls) -> str:
        return 'positive'


class PositiveIntegerField(IntegerField):

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return PositiveIntegerFieldOpenApiManager(self)

    def __init__(self, min_value: int = None, *args, **kwargs):
        if min_value is None or min_value < 1:
            min_value = 1
        super().__init__(min_value=min_value, *args, **kwargs)



# -*- coding: utf-8 -*-
import inspect
import abc
from typing import List, Any, Optional, Type, Dict

from a3py.improved.json import fast_loads
from django.core.exceptions import ValidationError
from django.http import FileResponse
from django.db.models import JSONField
from django.views.generic import View

from a3dj.core import inner_errors
from a3dj.core.response_utils import build_success_response, FileResponseStruct
from a3dj.structs import json_structs
from a3dj.structs.form_structs.base_form_struct import BaseFormStruct
from a3dj.structs.json_structs.base_json_struct import BaseJsonStruct
from a3dj.structs.mockers import FixedValueMocker
from a3dj.errors import CRequestStructError
from a3dj.structs.fields import ListField, ModelMultipleInstanceField

from .base_request_permission import BaseRequestPermission


class DynamicResponseStruct(json_structs.JsonStruct):

    class Meta(json_structs.JsonStructOptions):
        is_abstract = True


class BasePostViewMetaClass(abc.ABCMeta):

    def __new__(mcs, name, bases, attrs: dict, **kwargs):
        # exclude self
        parents = [b for b in bases if isinstance(b, BasePostViewMetaClass)]
        if not parents:
            return super().__new__(mcs, name, bases, attrs, **kwargs)

        # new class
        new_cls = super().__new__(mcs, name, bases, attrs, **kwargs)

        if inspect.isabstract(new_cls):
            # 复杂动态组装情况
            return new_cls

        r = new_cls.handle_business.__annotations__.get('return', None) # noqa
        if r == DynamicResponseStruct and getattr(new_cls, 'data_response_struct_cls', None) is None:
            # 直接让data_response_struct_cls成为DynamicResponseStruct的情况
            return new_cls

        if inspect.isclass(r) and issubclass(r, FileResponse):
            new_cls.response_struct_cls = FileResponseStruct
            new_cls.data_response_struct_cls = None
            return new_cls

        # dynamic build response struct
        class SuccessResponseStruct(json_structs.JsonStruct):
            status = json_structs.CharField(mocker=FixedValueMocker('OK'), default='OK')

        response_struct_cls = SuccessResponseStruct
        data_response_struct_cls = None

        def _check_data_response_struct_cls(data_cls: Type[json_structs.JsonStruct]):
            if len(data_cls.get_meta().fields) == 0:
                raise inner_errors.ViewError(f"Invalid view {new_cls.__name__} response data struct: empty fields.")

        if r is None:
            pass
        elif issubclass(r, DynamicResponseStruct):
            data_response_struct_cls = new_cls.dynamic_build_response_struct() # noqa
            _check_data_response_struct_cls(data_response_struct_cls)
            response_struct_cls.dynamic_add_field('data', json_structs.ObjectField(obj_cls=data_response_struct_cls))

        elif issubclass(r, (json_structs.JsonStruct, json_structs.ModelJsonStruct)):
            data_response_struct_cls = r
            _check_data_response_struct_cls(data_response_struct_cls)
            response_struct_cls.dynamic_add_field('data', json_structs.ObjectField(obj_cls=data_response_struct_cls))

        else:
            raise inner_errors.ViewError(f"Invalid view {new_cls.__name__} response struct type: {r.__name__}")

        new_cls.response_struct_cls = response_struct_cls
        new_cls.data_response_struct_cls = data_response_struct_cls
        return new_cls


class BasePostView(View, metaclass=BasePostViewMetaClass):
    response_struct_cls: json_structs.JsonStruct
    data_response_struct_cls: json_structs.JsonStruct

    request_permissions: List[BaseRequestPermission] = None
    request_struct_cls = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._request_data: Optional[Dict] = None

    @abc.abstractmethod
    def handle_business(self, request_struct, custom_params):
        raise NotImplementedError()

    def post(self, *_, **__):
        for permission in self.request_permissions or list():
            permission(self.request)

        request_struct = self.validate_request_struct()
        custom_params = self.custom_validate(request_struct)

        r = self.handle_business(request_struct, custom_params)

        if r is None:
            return build_success_response()
        elif isinstance(r, BaseJsonStruct):
            return build_success_response(data=r.to_json_dict())
        elif isinstance(r, FileResponse):
            return r
        else:
            raise inner_errors.ViewError(f"Invalid response type: {type(r).__name__}")

    def custom_validate(self, request_struct) -> Any:
        pass

    def _get_json_request_data(self) -> dict:
        try:
            return fast_loads(self.request.body)
        except Exception as __:
            raise CRequestStructError(JSONField.default_error_messages['invalid'])

    def _get_form_request_data(self):
        return self.request.POST

    def _get_form_request_files(self):
        return self.request.FILES

    def _get_request_data(self) -> dict:
        if self._request_data is not None:
            return self._request_data

        d = dict()
        if self.request_struct_cls is not None and inspect.isclass(self.request_struct_cls):
            if issubclass(self.request_struct_cls, BaseJsonStruct):
                d.update(self._get_json_request_data() or dict())
            elif issubclass(self.request_struct_cls, BaseFormStruct):
                d.update(self._get_form_request_data() or dict())
                d.update(self._get_form_request_files() or dict())

                for field_name, field_instance in self.request_struct_cls.get_meta().fields.items():
                    vl = d.get(field_name, None)
                    if vl is None:
                        continue

                    if isinstance(field_instance, (ListField, ModelMultipleInstanceField)):
                        d[field_name] = vl
                    else:
                        d[field_name] = vl[0]

            else:
                raise inner_errors.ViewError(f"Invalid request struct type: {self.request_struct_cls.__name__}")

        self._request_data = d
        return self._request_data

    @classmethod
    def _full_clean_request_struct(cls, request_struct):
        try:
            request_struct.full_clean()
        except ValidationError as e:
            raise CRequestStructError(str(e), repr(e))

    def validate_request_struct(self) -> Optional[json_structs.JsonStruct]:
        request_struct = None

        if self.request_struct_cls is not None and inspect.isclass(self.request_struct_cls):
            data = self._get_request_data()
            request_struct = self.request_struct_cls(data)
            self._full_clean_request_struct(request_struct)

        return request_struct

    @classmethod
    def dynamic_build_response_struct(cls) -> Type[json_structs.JsonStruct]:
        pass



# -*- coding: utf-8 -*-
from typing import List, Tuple

from django.core.exceptions import ValidationError
from django.db.models.fields import Field


def validate_choices(choices: List[Tuple], value):
    for option_key, option_value in choices:
        if value == option_key:
            return value
    raise ValidationError(
        message=Field.default_error_messages["invalid_choice"],
        code="invalid_choice",
        params={"value": value},
    )


def choices2open_api_dict(choices: List[Tuple]) -> dict:
    enum_list = list()
    enum_dict = dict()
    for option_key, option_value in choices:
        enum_list.append(option_key)
        enum_dict[option_key] = option_value

    return {
        'enum': enum_list,
        'x-apifox': {
            'enumDescriptions': enum_dict
        }
    }



# -*- coding: utf-8 -*-
from typing import Optional
from .base_mocker import BaseMocker


class IntegerMocker(BaseMocker):

    def __init__(self, min_value: int = None, max_value: int = None, choices=None):
        super().__init__()

        self._choices = None
        self._choice_index = 0
        self._min_value = None
        self._max_value = None

        if choices is not None:
            self._choices = list()
            for k, _ in choices:
                self._choices.append(k)
        else:
            if max_value is None and min_value is None:
                min_value = 500
                max_value = 1500
            elif max_value is not None:
                min_value = max_value - 500
                if max_value > 0 and min_value < 1:
                    min_value = 1
            else:
                max_value = min_value + 1000

            self._max_value = max_value
            self._min_value = min_value

    def mock(self, _, __) -> str:
        if self._choices is not None:
            v = self._choices[self._choice_index]
            self._choice_index += 1
            if self._choice_index >= len(self._choices):
                self._choice_index = 0
            return v

        return self.f.random_int(self._min_value, self._max_value)

    def to_mock_string(self) -> Optional[str]:
        if self._choices is not None:
            return
        return f"@integer({self._min_value}, {self._max_value})"

    def to_json_mock_string(self) -> Optional[str]:
        if self._choices is not None:
            return f"{self._choices[0][0]}"
        return f"@integer({self._min_value}, {self._max_value})"

    def to_form_mock_string(self) -> Optional[str]:
        if self._choices is not None:
            return f"{self._choices[0][0]}"
        return f"{{% mock 'integer' ,  {self._min_value}, {self._max_value} %}}"



# -*- coding: utf-8 -*-
import abc
from collections import defaultdict
from typing import Optional

from a3faker import FakerProxy

_GlobalSequenceDict = defaultdict(int)


class BaseMocker(abc.ABC):

    def __init__(self):
        self.f = FakerProxy.get_faker()
        self._gsd = _GlobalSequenceDict
        self._cls_name = self.__class__.__name__

    def _get_sequence_index(self) -> int:
        self._gsd[self._cls_name] += 1
        return self._gsd[self._cls_name]

    @abc.abstractmethod
    def to_json_mock_string(self) -> Optional[str]:
        raise NotImplementedError()

    @abc.abstractmethod
    def to_form_mock_string(self) -> Optional[str]:
        raise NotImplementedError()

    @abc.abstractmethod
    def mock(self, instance, root_instance):
        # 用于单元测试时mock http的request
        raise NotImplementedError()



# -*- coding: utf-8 -*-
from a3dj.utils.hint_utils import TypeA3Model

from ..model_struct import ModelStructMetaClass, ModelStructMixin, ModelStructOptions
from .base_form_struct import BaseFormStruct
from .form_struct import FormStructMetaClass


ModelFormStructOptions = ModelStructOptions


class ModelFormStructMetaClass(ModelStructMetaClass):
    _check_field = FormStructMetaClass._check_field # noqa


class ModelFormStruct(BaseFormStruct, ModelStructMixin, metaclass=ModelFormStructMetaClass):
    _meta: ModelStructOptions

    def __init__(self, data=None, instance: TypeA3Model = None, **kwargs):
        BaseFormStruct.__init__(self, data=data, **kwargs)
        ModelStructMixin.__init__(self, instance)

    def clean(self):
        ModelStructMixin.clean(self)



# -*- coding: utf-8 -*-
from typing import List
from django.utils.translation import gettext_lazy as _
from a3dj.core.inner_errors import MockError

from .base_mocker import BaseMocker


class ListMocker(BaseMocker):
    default_error_messages = {
        "NotEnough": _("Can not fake enough items, min length is %(min_length)s, current list %(str_list)s."),
    }

    def __init__(self, item_type, min_length: int = None, max_length: int = None, unique: bool = False):
        super().__init__()

        self.item_type = item_type
        self.unique = unique

        self._min_length = 1
        self._max_length = 3

        if min_length is not None and min_length > self._min_length:
            self._min_length = min_length
            self._max_length = self._min_length + 2

        if max_length is not None and max_length < self._max_length:
            self._max_length = max_length

    def mock(self, instance, root_instance) -> List:
        length = self.f.random_int(self._min_length, self._max_length)
        rl = list()
        for i in range(length):
            v = self.item_type.mock_manager.mock(instance=instance, root_instance=root_instance)
            if self.unique and v in rl:
                continue

            rl.append(v)
        if len(rl) < self._min_length:
            raise MockError(
                error_message=self.default_error_messages['NotEnough'],
                params={
                    'min_length': self._min_length,
                    'str_list': str(rl)
                }
            )
        return rl

    def to_json_mock_string(self):
        return

    def to_form_mock_string(self):
        v = self.item_type.mock_manager.to_form_mock_string()
        if v not in ['', None]:
            return [v, ]



# -*- coding: utf-8 -*-
from .base_json_struct import BaseJsonStruct
from .json_struct import JsonStruct, JsonStructOptions
from .model_json_struct import ModelJsonStruct, ModelJsonStructOptions
from ..fields import BooleanField, CharField, IntegerField, DateField, TimeField, DateTimeField, DurationField, \
    DecimalField, EmailField, FloatField, GenericIPAddressField, PositiveIntegerField, URLField, UUIDField, \
    JSONField, BinaryField, FileField, ImageField, ListField, \
    ObjectField



# -*- coding: utf-8 -*-
import inspect
from abc import ABC
from itertools import chain
from typing import Dict, Tuple, Any, List, Type, Set
from django.db import models

from a3dj.utils.hint_utils import TypeA3Model
from a3dj.core.inner_errors import ModelStructError
from a3dj.structs.base.struct import StructMetaClass, StructOptions

from .model_field_translater import ModelFieldTranslater


class ModelStructOptions(StructOptions):
    model: Type[TypeA3Model] = None
    include: List[str] = None
    exclude: List[str] = None

    def __init__(self):
        self.m2m_field_list = list()
        self.cache_model_fields = dict()
        self.include_field_set = set()
        self.exclude_field_set = set()


class ModelStructMetaClass(StructMetaClass, ABC):

    def _get_meta(cls, meta: Type[ModelStructOptions]) -> ModelStructOptions:
        if inspect.isclass(meta) and issubclass(meta, ModelStructOptions):
            return meta()
        raise ModelStructError(f"{cls.__name__} must have correct ModelOptions")

    def _add_other_fields(cls, field_name_set: Set[str]):
        # add model fields
        meta: ModelStructOptions = cls._meta
        opts = getattr(meta.model, '_meta')
        for f in chain(opts.concrete_fields, opts.private_fields, opts.many_to_many):
            if meta.include and f.name not in meta.include:
                meta.exclude_field_set.add(f.name)
                continue
            if meta.exclude and f.name in meta.exclude:
                meta.exclude_field_set.add(f.name)
                continue
            if f.name in field_name_set and meta.fields.get(f.name, None) is None:
                # 子类屏蔽
                meta.exclude_field_set.add(f.name)
                continue

            field_name_set.add(f.name)
            meta.include_field_set.add(f.name)
            meta.cache_model_fields[f.name] = f

            if isinstance(f, models.ManyToManyField):
                meta.m2m_field_list.append(f.name)

            # field转换
            meta.fields[f.name] = ModelFieldTranslater.translate(f)

    def _check_meta(cls):
        meta: ModelStructOptions = cls._meta

        if not inspect.isclass(meta.model) or not issubclass(meta.model, models.Model):
            raise ModelStructError(f"{cls.__name__} must set correct model, not {type(meta.model)}")

        if meta.include is not None and meta.exclude is not None:
            raise ModelStructError(f"{cls.__name__} must specify include or exclude, not both")


class ModelStructMixin:
    _meta: ModelStructOptions

    def __init__(self, instance: TypeA3Model = None):
        if instance is None:
            self.instance: TypeA3Model = self._meta.model()
            self._is_new = True
        else:
            self.instance = instance
            self._is_new = False

        self._has_save_instance: bool = False
        self._changed_data: dict = dict()

    def _apply_instance(self):
        for field_name in self._meta.include_field_set:
            model_field_instance = self._meta.cache_model_fields[field_name]
            field_instance = self._meta.fields[field_name]

            new_value = getattr(self, field_name)
            if not self._is_new:
                old_value = getattr(self.instance, field_name)

                if isinstance(model_field_instance, (models.BinaryField, models.FileField)) and \
                        not field_instance.required:
                    # update 时，不上传文件，不意味着删除
                    continue
                else:
                    if field_name in self._meta.m2m_field_list:
                        old_value = old_value.all()
                    has_changed, ov, nv = self._meta.fields[field_name].check_has_changed(old_value, new_value)
                    if has_changed:
                        self._changed_data[field_name] = (ov, nv)

            if field_name not in self._meta.m2m_field_list:
                model_field_instance.save_form_data(self.instance, new_value)

    def clean(self):
        self._apply_instance()
        self.instance.full_clean(exclude=self._meta.exclude_field_set)

    def save(self, commit=True) -> TypeA3Model:
        if commit:
            self.instance.save()
            self._save_m2m()
        self._has_save_instance = True
        return self.instance

    def _save_m2m(self):
        for field_name in self._meta.m2m_field_list:
            value = getattr(self, field_name)

            field_instance = self._meta.cache_model_fields[field_name]
            field_instance.save_form_data(self.instance, value)

    def save_m2m(self):
        if not self._has_save_instance:
            self.instance.save()
        self._save_m2m()

    def has_changed(self) -> bool:
        return len(self._changed_data) > 0

    def get_changed_data(self) -> Dict[str, Tuple[Any, Any]]:
        return self._changed_data



# -*- coding: utf-8 -*-
from typing import Tuple, Any, Type
from django.db import models
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

from a3dj.utils.hint_utils import TypeA3Model
from a3dj.structs.base.base_field import BaseField, BaseFieldMockManager, BaseFieldOpenApiManager
from .related_field_translater import RelatedFieldTranslater


class ModelInstanceField(BaseField):
    default_error_messages = {
        'not_exist': _("Model instance “%(value)s“ does not exist")
    }

    def __init__(
            self, model_cls: Type[TypeA3Model], to_field_name: str = None, query: dict = None,
            *args, **kwargs
    ):
        self.model_cls = model_cls
        self.to_field_name = to_field_name or 'id'
        self.query = query or dict()
        field_instance = getattr(model_cls, '_meta').get_field(self.to_field_name)
        self.outward_field: BaseField = RelatedFieldTranslater.translate(field_instance)

        super().__init__(*args, **kwargs)

    @classmethod
    def _build_from_model_field(cls, field_instance: models.ForeignKey) -> 'ModelInstanceField':
        return cls(model_cls=field_instance.related_model, to_field_name=field_instance.target_field.name)

    def to_python(self, value):
        v = self.outward_field.to_python(value)
        fd = {self.to_field_name: v}
        fd.update(self.query)

        instance = self.model_cls.objects.filter(**fd).first()
        if instance is None:
            raise ValidationError(
                message=self.default_error_messages["not_exist"],
                code="not_exist",
                params={"value": value}
            )

        return instance

    def check_has_changed(self, old_value, new_value) -> Tuple[bool, Any, Any]:
        ov = getattr(old_value, self.to_field_name, None)
        nv = getattr(new_value, self.to_field_name, None)
        if ov == nv:
            return False, None, None
        else:
            return True, ov, nv

    def to_json(self, value):
        return getattr(value, self.to_field_name, None)

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return self.outward_field.open_api_manager

    def _preset_mock_manager(self) -> BaseFieldMockManager:
        return self.outward_field.mock_manager

    @classmethod
    def support_json(cls) -> bool:
        return True

    @classmethod
    def support_form(cls) -> bool:
        return True



# -*- coding: utf-8 -*-
from typing import Dict, Type
from django.db import models

from a3dj.structs.base.base_field import BaseField
from a3dj.structs.fields.related_field_translater import RelatedFieldTranslater
from a3dj.structs import fields


class ModelFieldTranslater(RelatedFieldTranslater):
    cached_mappings = dict()
    mappings: Dict[Type[models.Field], Type[BaseField]] = {
        models.AutoField: fields.IntegerField,
        models.BigAutoField: fields.IntegerField,
        models.BigIntegerField: fields.IntegerField,
        models.BinaryField: fields.BinaryField,
        models.BooleanField: fields.BooleanField,
        models.CharField: fields.CharField,
        models.DateField: fields.DateField,
        models.DateTimeField: fields.DateTimeField,
        models.DecimalField: fields.DecimalField,
        models.DurationField: fields.DurationField,
        models.EmailField: fields.EmailField,
        models.FileField: fields.FileField,
        models.FilePathField: fields.CharField,
        models.FloatField: fields.FloatField,
        models.GenericIPAddressField: fields.GenericIPAddressField,
        models.ImageField: fields.ImageField,
        models.IntegerField: fields.IntegerField,
        models.JSONField: fields.JSONField,
        models.PositiveIntegerField: fields.PositiveIntegerField,
        models.PositiveBigIntegerField: fields.PositiveIntegerField,
        models.PositiveSmallIntegerField: fields.PositiveIntegerField,
        models.SlugField: fields.CharField,
        models.SmallAutoField: fields.IntegerField,
        models.SmallIntegerField: fields.IntegerField,
        models.TextField: fields.CharField,
        models.TimeField: fields.TimeField,
        models.URLField: fields.URLField,
        models.UUIDField: fields.UUIDField,
        # related
        models.ForeignKey: fields.ModelInstanceField,
        models.OneToOneField: fields.ModelInstanceField,
        models.ManyToManyField: fields.ModelMultipleInstanceField,
    }



# -*- coding: utf-8 -*-
import datetime
from typing import Optional

from .base_mocker import BaseMocker


class DurationMocker(BaseMocker):

    def to_json_mock_string(self) -> Optional[str]:
        return "@date"

    def to_form_mock_string(self) -> Optional[str]:
        return "{% mock 'date' %}"

    def mock(self, _, __) -> datetime.timedelta:
        first = self.f.date_time_this_year(before_now=False, after_now=True)
        second = self.f.date_time_this_year(before_now=True, after_now=False)
        return first - second



# -*- coding: utf-8 -*-
from a3dj.core.validators import ImageFileValidator

from .file_field import FileField


class ImageField(FileField):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.validators.append(ImageFileValidator())



# -*- coding: utf-8 -*-
from .base_mocker import BaseMocker
from .boolean_mocker import BooleanMocker
from .char_mocker import CharMocker
from .integer_mocker import IntegerMocker
from .date_mocker import DateMocker
from .time_mocker import TimeMocker
from .datetime_mocker import DateTimeMocker
from .duration_mocker import DurationMocker
from .decimal_mocker import DecimalMocker
from .email_mocker import EmailMocker
from .float_mocker import FloatMocker
from .generic_ip_address_mocker import GenericIPAddressMocker
from .url_mocker import URLMocker
from .uuid_mocker import UUIDMocker
from .none_mocker import NoneMocker

from .list_mocker import ListMocker
from .object_mocker import ObjectMocker

from .fixed_value_mocker import FixedValueMocker



# -*- coding: utf-8 -*-
from decimal import Decimal, DecimalException

from django.core.exceptions import ValidationError
from django.db.models.fields import DecimalField as DjangoDecimalField

from a3dj.structs.base.base_field import BaseField, BaseFieldMockManager, BaseFieldOpenApiManager, JsonType, FormType
from a3dj.structs.mockers import DecimalMocker, BaseMocker


class DecimalFieldOpenApiManager(BaseFieldOpenApiManager):

    @classmethod
    def _preset_form_type(cls) -> str:
        return FormType.String

    @classmethod
    def _preset_open_api_format(cls) -> str:
        return 'decimal'

    @classmethod
    def _preset_json_type(cls) -> str:
        return JsonType.String


class DecimalFieldMockManager(BaseFieldMockManager):

    def _preset_default_mocker(self) -> BaseMocker:
        return DecimalMocker()


class DecimalField(BaseField):

    @classmethod
    def support_form(cls) -> bool:
        return True

    @classmethod
    def support_json(cls) -> bool:
        return True

    @classmethod
    def _build_from_model_field(cls, field_instance: DjangoDecimalField) -> 'DecimalField':
        return cls()

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return DecimalFieldOpenApiManager(self)

    def _preset_mock_manager(self) -> BaseFieldMockManager:
        return DecimalFieldMockManager(self)

    def to_python(self, value) -> Decimal:
        if isinstance(value, Decimal):
            return value

        try:
            return Decimal(str(value))
        except DecimalException:
            raise ValidationError(
                DjangoDecimalField.default_error_messages["invalid"],
                code="invalid",
                params={"value": value},
            )

    def to_json(self, value: Decimal) -> str:
        return str(value)



# -*- coding: utf-8 -*-
import json
from typing import Any, Tuple

from django.core.exceptions import ValidationError
from django.db.models import JSONField as ModelJSONField

from a3dj.structs.base.base_field import BaseFieldMockManager, BaseFieldOpenApiManager, JsonType, FormType
from a3dj.structs.mockers import NoneMocker, BaseMocker

from .char_field import CharField


class JSONFieldOpenApiManager(BaseFieldOpenApiManager):

    @classmethod
    def _preset_form_type(cls) -> str:
        return FormType.String

    @classmethod
    def _preset_open_api_format(cls) -> str:
        return 'json'

    @classmethod
    def _preset_json_type(cls) -> str:
        return JsonType.String


class JSONFieldMockManager(BaseFieldMockManager):

    def _preset_default_mocker(self) -> BaseMocker:
        return NoneMocker()


class JSONField(CharField):

    def __init__(self, encoder=None, decoder=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.encoder = encoder
        self.decoder = decoder

    @classmethod
    def _build_from_model_field(cls, field_instance: ModelJSONField) -> 'CharField':
        return cls(
            max_length=field_instance.max_length, choices=field_instance.choices,
            encoder=field_instance.encoder, decoder=field_instance.decoder
        )

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return JSONFieldOpenApiManager(self)

    def _preset_mock_manager(self) -> BaseFieldMockManager:
        return JSONFieldMockManager(self)

    def to_python(self, value) -> Any:
        value = super().to_python(value)

        try:
            return json.loads(value, cls=self.decoder)
        except json.JSONDecodeError:
            raise ValidationError(
                ModelJSONField.default_error_messages["invalid"],
                code="invalid"
            )

    def to_json(self, value) -> str:
        return json.dumps(value, ensure_ascii=False, cls=self.encoder)

    def check_has_changed(self, old_value, new_value) -> Tuple[bool, Any, Any]:
        ov = self.to_json(old_value)
        nv = self.to_json(new_value)
        return super().check_has_changed(ov, nv)



# -*- coding: utf-8 -*-
from a3dj.structs.base.base_field import BaseField
from a3dj.structs.base.struct import StructMetaClass, StructOptions
from a3dj.core.inner_errors import FieldError
from .base_json_struct import BaseJsonStruct


JsonStructOptions = StructOptions


class JsonStructMetaClass(StructMetaClass):

    def _check_field(cls, field_name, field_instance: BaseField):
        if not field_instance.support_json():
            raise FieldError(
                f"field: {field_name}, class: {field_instance.__class__.__name__}, not support json struct."
            )


class JsonStruct(BaseJsonStruct, metaclass=JsonStructMetaClass):
    pass



# -*- coding: utf-8 -*-
from typing import Optional

from .base_mocker import BaseMocker


class URLMocker(BaseMocker):

    def to_json_mock_string(self) -> Optional[str]:
        return '@url("http")'

    def to_form_mock_string(self) -> Optional[str]:
        return "{% mock 'url' , 'http' %}"

    def mock(self, _, __) -> str:
        return self.f.url()



# -*- coding: utf-8 -*-
import datetime

from django.core.exceptions import ValidationError
from django.db.models.fields import DateField as DjangoDateField

from a3py.simplified.datetime import date2str
from a3dj.structs.base.base_field import BaseField, BaseFieldMockManager, BaseFieldOpenApiManager, JsonType, FormType
from a3dj.structs.mockers import DateMocker, BaseMocker


class DateFieldOpenApiManager(BaseFieldOpenApiManager):

    @classmethod
    def _preset_form_type(cls) -> str:
        return FormType.String

    @classmethod
    def _preset_open_api_format(cls) -> str:
        return 'date'

    @classmethod
    def _preset_json_type(cls) -> str:
        return JsonType.String


class DateFieldMockManager(BaseFieldMockManager):

    def _preset_default_mocker(self) -> BaseMocker:
        return DateMocker()


class DateField(BaseField):

    @classmethod
    def support_form(cls) -> bool:
        return True

    @classmethod
    def support_json(cls) -> bool:
        return True

    @classmethod
    def _build_from_model_field(cls, field_instance: DjangoDateField) -> 'DateField':
        return cls()

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return DateFieldOpenApiManager(self)

    def _preset_mock_manager(self) -> BaseFieldMockManager:
        return DateFieldMockManager(self)

    def to_python(self, value) -> datetime.date:
        if isinstance(value, datetime.date):
            return value

        try:
            return datetime.date.fromisoformat(str(value))
        except ValueError:
            raise ValidationError(
                DjangoDateField.default_error_messages["invalid_date"],
                code="invalid_date",
                params={"value": value},
            )

    def to_json(self, value: datetime.date) -> str:
        return date2str(value)



# -*- coding: utf-8 -*-
from typing import List, Tuple

from django.core.exceptions import ValidationError
from django.core.validators import MinValueValidator, MaxValueValidator
from django.db.models import IntegerField as ModelIntegerField

from a3dj.structs.base.base_field import BaseField, BaseFieldMockManager, BaseFieldOpenApiManager, JsonType, FormType
from a3dj.structs.mockers import IntegerMocker, BaseMocker
from a3dj.core.utils import set_meaning_kv

from .utils.choices_utils import validate_choices, choices2open_api_dict


TypeIntegerChoices = List[Tuple[int, str]]


class IntegerFieldOpenApiManager(BaseFieldOpenApiManager):
    field: 'IntegerField'

    @classmethod
    def _preset_form_type(cls) -> str:
        return FormType.Integer

    @classmethod
    def _preset_open_api_format(cls) -> str:
        return 'long'

    @classmethod
    def _preset_json_type(cls) -> str:
        return JsonType.Integer

    def _build_json_base_info(self) -> dict:
        info = super()._build_json_base_info()
        set_meaning_kv(info, 'minimum', self.field.min_value)
        set_meaning_kv(info, 'maximum', self.field.max_value)

        if self.field.choices is not None:
            d = choices2open_api_dict(choices=self.field.choices)
            info.update(d)

        return info


class IntegerFieldMockManager(BaseFieldMockManager):
    field: 'IntegerField'

    def _preset_default_mocker(self) -> BaseMocker:
        return IntegerMocker(
            min_value=self.field.min_value, max_value=self.field.max_value, choices=self.field.choices
        )


class IntegerField(BaseField):

    @classmethod
    def support_form(cls) -> bool:
        return True

    @classmethod
    def support_json(cls) -> bool:
        return True

    @classmethod
    def _build_from_model_field(cls, field_instance: ModelIntegerField) -> 'IntegerField':
        return cls(choices=field_instance.choices)

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return IntegerFieldOpenApiManager(self)

    def _preset_mock_manager(self) -> BaseFieldMockManager:
        return IntegerFieldMockManager(self)

    def __init__(
            self, min_value: int = None, max_value: int = None, choices: TypeIntegerChoices = None, *args, **kwargs
    ):
        super().__init__(*args, **kwargs)

        self.min_value = min_value
        self.max_value = max_value
        self.choices = choices

        if self.min_value is not None:
            self.validators.append(MinValueValidator(self.min_value))

        if self.max_value is not None:
            self.validators.append(MaxValueValidator(self.max_value))

    def validate(self, value, struct_instance):
        value = super().validate(value, struct_instance)

        if self.choices is not None and value is not None:
            value = validate_choices(choices=self.choices, value=value)

        return value

    def to_python(self, value) -> int:
        if isinstance(value, int):
            return value

        try:
            return int(value)
        except (TypeError, ValueError):
            raise ValidationError(
                message=ModelIntegerField.default_error_messages["invalid"],
                code="invalid",
                params={"value": value},
            )

    def to_json(self, value: int) -> int:
        return value



# -*- coding: utf-8 -*-
from django.core import validators

from a3dj.structs.base.base_field import BaseFieldMockManager, BaseFieldOpenApiManager, JsonType, FormType
from a3dj.structs.mockers import EmailMocker, BaseMocker

from .char_field import CharField


class EmailFieldOpenApiManager(BaseFieldOpenApiManager):

    @classmethod
    def _preset_form_type(cls) -> str:
        return FormType.String

    @classmethod
    def _preset_open_api_format(cls) -> str:
        return 'email'

    @classmethod
    def _preset_json_type(cls) -> str:
        return JsonType.String


class EmailFieldMockManager(BaseFieldMockManager):

    def _preset_default_mocker(self) -> BaseMocker:
        return EmailMocker()


class EmailField(CharField):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.validators.append(validators.validate_email)

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return EmailFieldOpenApiManager(self)

    def _preset_mock_manager(self) -> BaseFieldMockManager:
        return EmailFieldMockManager(self)



# -*- coding: utf-8 -*-
from django.core.exceptions import ValidationError
from django.core.files import File
from django.utils.translation import gettext_lazy as _
from django.utils.deconstruct import deconstructible

from a3dj.utils.media_utils import PillowImage
from a3py.improved.readable import get_readable_size


@deconstructible
class FileMaxSizeValidator:
    messages = {
        "invalid": _("File max size must less than %(value)s.")
    }

    def __init__(self, max_length: int):
        self.max_length = max_length
        self._readable_size = get_readable_size(self.max_length)

    def __call__(self, f: File):
        if len(f) > self.max_length:
            raise ValidationError(
                self.messages["invalid"], code="invalid", params={"value": self._readable_size}
            )


@deconstructible
class ImageFileValidator:
    messages = {
        "invalid": _("It is not a valid image.")
    }

    def __call__(self, f: File):
        try:
            img = PillowImage(file=f)
            f.image = img.image
            f.content_type = img.get_content_type()
        except Exception as _:
            raise ValidationError(
                self.messages["invalid"], code="invalid"
            )



# -*- coding: utf-8 -*-
from .boolean_field import BooleanField
from .char_field import CharField
from .integer_field import IntegerField
from .date_field import DateField
from .time_field import TimeField
from .datetime_field import DateTimeField
from .duration_field import DurationField
from .decimal_field import DecimalField
from .email_field import EmailField
from .float_field import FloatField
from .generic_ip_address_field import GenericIPAddressField
from .positive_integer_field import PositiveIntegerField
from .url_field import URLField
from .uuid_field import UUIDField
from .json_field import JSONField
from .binary_field import BinaryField
from .file_field import FileField
from .image_field import ImageField

from .list_field import ListField
from .object_field import ObjectField
from .model_instance_field import ModelInstanceField
from .model_multiple_instance_field import ModelMultipleInstanceField



# -*- coding: utf-8 -*-
import abc
from typing import List, Callable, Any, Optional, Tuple

from django.core.exceptions import ValidationError
from django.forms.fields import Field as DjangoFormField
from django.db.models import Field as ModelField, NOT_PROVIDED

from a3dj.structs.mockers import BaseMocker
from a3dj.core.utils import set_meaning_kv


class FormType:
    String = "string"
    Integer = 'integer'
    Number = 'number'
    Array = 'array'
    File = 'file'


class JsonType:
    Boolean = 'boolean'
    String = 'string'
    Integer = 'integer'
    Number = 'number'
    Array = 'array'
    Object = 'object'


_HAS_DEFAULT_JSON_TYPES = (JsonType.Boolean, JsonType.String, JsonType.Integer, JsonType.Number)


class BaseFieldOpenApiManager(abc.ABC):

    def __init__(self, field: "BaseField"):
        self.field = field

    @classmethod
    @abc.abstractmethod
    def _preset_open_api_format(cls) -> str:
        raise NotImplementedError()

    @classmethod
    @abc.abstractmethod
    def _preset_json_type(cls) -> str:
        raise NotImplementedError()

    @classmethod
    @abc.abstractmethod
    def _preset_form_type(cls) -> str:
        raise NotImplementedError()

    def get_components(self) -> Optional[dict]:
        return

    def _build_json_base_info(self) -> dict:
        info = {
            'type': self._preset_json_type(),
            'format': self._preset_open_api_format(),
        }

        set_meaning_kv(info, 'title', self.field.verbose_name)
        set_meaning_kv(info, 'description', self.field.description)

        if self._preset_json_type() in _HAS_DEFAULT_JSON_TYPES and self.field.default is not None:
            if isinstance(self.field.default, Callable):
                value = self.field.default()
            else:
                value = self.field.default

            info['default'] = self.field.to_json(value)

        mock_value = self.field.mock_manager.to_json_mock_string()
        if mock_value not in [None, '']:
            info['mock'] = {
                'mock': mock_value
            }
        return info

    def _build_form_base_info(self) -> dict:
        form_type = self._preset_form_type()
        if form_type == FormType.File:
            info = {
                "type": FormType.String,
                "format": "binary"
            }
        else:
            info = {
                'type': form_type
            }

        dl = list()
        if self.field.verbose_name is not None:
            dl.append(self.field.verbose_name)
        if self.field.description not in [None, '']:
            dl.append(self.field.description)

        set_meaning_kv(info, 'description', ';'.join(dl))

        mock_value = self.field.mock_manager.to_form_mock_string()
        if mock_value not in [None, '']:
            info['example'] = mock_value
        return info

    def to_json_open_api_dict(self) -> dict:
        return self._build_json_base_info()

    def to_form_open_api_dict(self) -> dict:
        return self._build_form_base_info()


class BaseFieldMockManager(abc.ABC):

    def __init__(self, field: "BaseField"):
        self.field = field

    def mock(self, instance, root_instance) -> Any:
        mocker = self._get_mocker()
        return mocker.mock(instance, root_instance)

    def to_json_mock_string(self):
        mocker = self._get_mocker()
        return mocker.to_json_mock_string()

    def to_form_mock_string(self):
        mocker = self._get_mocker()
        return mocker.to_form_mock_string()

    def _get_mocker(self) -> BaseMocker:
        if self.field.mocker is None:
            self.field.mocker = self._preset_default_mocker()
        return self.field.mocker

    @abc.abstractmethod
    def _preset_default_mocker(self) -> BaseMocker:
        raise NotImplementedError()


class BaseField(abc.ABC):

    def __init__(
            self,
            verbose_name=None,
            description=None,
            default=None,
            required: bool = True,
            validators: List = None,
            mocker: BaseMocker = None
    ):
        self.verbose_name = verbose_name
        self.description = description or ''
        self.default = default
        self.required = required
        self.validators = validators or list()
        self.mocker = mocker

        self.open_api_manager = self._preset_open_api_manager()
        self.mock_manager = self._preset_mock_manager()

        self._struct_cls = None
        self._name = None

    def contribute_to_struct(self, struct_cls, name: str):
        self._struct_cls = struct_cls
        self._name = name

    def __str__(self) -> str:
        if self._struct_cls is None:
            return super().__str__()
        return f'{self._struct_cls}.{self._name}'

    def __repr__(self):
        path = f'{self.__class__.__module__}.{self.__class__.__qualname__}'
        if self._name is None:
            return f'<{path}>'
        return f'<{path}: {self._name}>'

    def _run_validators(self, value):
        for v in self.validators:
            v(value)

    def validate(self, value, struct_instance):
        if value is None and self.default is not None:
            if isinstance(self.default, Callable):
                value = self.default()
            else:
                value = self.default

        if value is None and self.required:
            raise ValidationError(message=DjangoFormField.default_error_messages["required"], code="required")

        return value

    def clean_for_python(self, value, struct_instance):
        value = self.validate(value, struct_instance)
        if value is None:
            return value

        value = self.to_python(value)
        self._run_validators(value)
        return value

    def clean_for_json(self, value, struct_instance):
        if value is not None:
            value = self.to_json(value)
        return value

    @classmethod
    @abc.abstractmethod
    def support_form(cls) -> bool:
        raise NotImplementedError()

    @classmethod
    @abc.abstractmethod
    def support_json(cls) -> bool:
        raise NotImplementedError()

    @classmethod
    def build_from_model_field(cls, field_instance: ModelField) -> 'BaseField':
        instance = cls._build_from_model_field(field_instance)
        instance.verbose_name = field_instance.verbose_name
        instance.description = field_instance.help_text
        instance.required = not field_instance.null

        if field_instance.default is NOT_PROVIDED:
            instance.default = None
        else:
            instance.default = field_instance.default

        return instance

    def check_has_changed(self, old_value, new_value) -> Tuple[bool, Any, Any]:
        if old_value == new_value:
            return False, None, None
        else:
            return True, old_value, new_value

    @classmethod
    @abc.abstractmethod
    def _build_from_model_field(cls, field_instance: ModelField) -> 'BaseField':
        raise NotImplementedError()

    @abc.abstractmethod
    def to_python(self, value):
        raise NotImplementedError()

    @abc.abstractmethod
    def to_json(self, value):
        raise NotImplementedError()

    @abc.abstractmethod
    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        raise NotImplementedError()

    @abc.abstractmethod
    def _preset_mock_manager(self) -> BaseFieldMockManager:
        raise NotImplementedError()




# -*- coding: utf-8 -*-
import datetime
from django.conf import settings
from django.utils import timezone
from django.core.exceptions import ValidationError
from django.db.models.fields import DateTimeField as DjangoDateTimeField

from a3py.simplified.datetime import datetime2str
from a3dj.structs.base.base_field import BaseField, BaseFieldMockManager, BaseFieldOpenApiManager, JsonType, FormType
from a3dj.structs.mockers import DateTimeMocker, BaseMocker


class DateTimeFieldOpenApiManager(BaseFieldOpenApiManager):

    @classmethod
    def _preset_form_type(cls) -> str:
        return FormType.String

    @classmethod
    def _preset_open_api_format(cls) -> str:
        return 'datetime'

    @classmethod
    def _preset_json_type(cls) -> str:
        return JsonType.String


class DateTimeFieldMockManager(BaseFieldMockManager):

    def _preset_default_mocker(self) -> BaseMocker:
        return DateTimeMocker()


class DateTimeField(BaseField):

    @classmethod
    def support_form(cls) -> bool:
        return True

    @classmethod
    def support_json(cls) -> bool:
        return True

    @classmethod
    def _build_from_model_field(cls, field_instance: DjangoDateTimeField) -> 'DateTimeField':
        return cls()

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return DateTimeFieldOpenApiManager(self)

    def _preset_mock_manager(self) -> BaseFieldMockManager:
        return DateTimeFieldMockManager(self)

    def to_python(self, value) -> datetime.datetime:
        if isinstance(value, datetime.datetime):
            parsed = value
        else:
            try:
                parsed = datetime.datetime.fromisoformat(str(value))
            except ValueError:
                raise ValidationError(
                    DjangoDateTimeField.default_error_messages["invalid_datetime"],
                    code="invalid_datetime",
                    params={"value": value},
                )

        if settings.USE_TZ and not timezone.is_aware(parsed):
            parsed = timezone.make_aware(parsed)
        return parsed

    def to_json(self, value: datetime.datetime) -> str:
        return datetime2str(value)



# -*- coding: utf-8 -*-
from typing import Optional
from .base_mocker import BaseMocker


class CharMocker(BaseMocker):

    def __init__(self, min_length: int = None, max_length: int = None, choices=None):
        super().__init__()

        self._choices = None
        self._choice_index = 0
        self._min_length = None
        self._max_length = None

        if choices is not None:
            self._choices = list()
            for k, _ in choices:
                self._choices.append(k)
        else:
            if max_length is None and min_length is None:
                min_length = 5
                max_length = 15
            elif max_length is not None:
                min_length = max_length - 10
                if min_length < 1:
                    min_length = 1
            else:
                max_length = min_length + 10

            self._max_length = max_length
            self._min_length = min_length

    def mock(self, _, __) -> str:
        if self._choices is not None:
            v = self._choices[self._choice_index]
            self._choice_index += 1
            if self._choice_index >= len(self._choices):
                self._choice_index = 0
            return v

        return self.f.pystr(self._min_length, self._max_length)

    def to_json_mock_string(self) -> Optional[str]:
        if self._choices is not None:
            return f"{self._choices[0][0]}"
        return f"@string({self._min_length}, {self._max_length})"

    def to_form_mock_string(self) -> Optional[str]:
        if self._choices is not None:
            return f"{self._choices[0][0]}"
        return f"{{% mock 'string' ,  {self._min_length}, {self._max_length} %}}"



# -*- coding: utf-8 -*-
import uuid
from typing import Optional

from .base_mocker import BaseMocker


class UUIDMocker(BaseMocker):

    def to_json_mock_string(self) -> Optional[str]:
        return "@uuid"

    def to_form_mock_string(self) -> Optional[str]:
        return "{% mock 'uuid' %}"

    def mock(self, _, __) -> uuid.UUID:
        return uuid.uuid4()



# -*- coding: utf-8 -*-

class BaseInnerError(Exception):

    def __init__(
            self,
            error_message: str,
            params: dict = None,
    ):
        if params:
            error_message %= params

        self.error_message = error_message

    def __str__(self):
        return f'{self.error_message}'

    def __repr__(self):
        return f"{self.__class__.__name__}: <{self}>"


class MockError(BaseInnerError):
    pass


class FieldError(BaseInnerError):
    pass


class ModelStructError(BaseInnerError):
    pass


class ModelFieldTranslateError(BaseInnerError):
    pass


class ViewError(BaseInnerError):
    pass


class OpenApiDocError(BaseInnerError):
    pass


class MixingError(BaseInnerError):
    pass



# -*- coding: utf-8 -*-
from django.core.files import File
from django.core.files.temp import NamedTemporaryFile


def build_temp_file_from_file(file: File) -> NamedTemporaryFile:
    temp_file = NamedTemporaryFile()
    for chunk in file.chunks(1024 * 1024):
        temp_file.write(chunk)
    temp_file.flush()
    return temp_file


class PillowImage:

    def __init__(self, filename: str = None, file: File = None):
        from PIL import Image

        if filename is None:
            self._temp_file = build_temp_file_from_file(file)
            filename = self._temp_file.name

        self.image: Image.Image = Image.open(filename)
        self.image.verify()
        self._filename = filename
        self._image_module = Image

    def get_content_type(self):
        return self._image_module.MIME.get(self.image.format)


# class CV2Video:
#
#     def __init__(self, filename: str = None, file: File = None):
#         import cv2
#
#         if filename is None:
#             self._temp_file = build_temp_file_from_file(file)
#             filename = self._temp_file.name
#
#         self.video = cv2.VideoCapture(filename)
#         self._filename = filename
#         self._cv2 = cv2
#
#     def get_width(self) -> int:
#         return int(self.video.get(self._cv2.CAP_PROP_FRAME_WIDTH))
#
#     def get_height(self) -> int:
#         return int(self.video.get(self._cv2.CAP_PROP_FRAME_HEIGHT))
#
#     def get_fps(self) -> int:
#         return int(self.video.get(self._cv2.CAP_PROP_FPS))
#
#     def get_frame_count(self) -> int:
#         return int(self.video.get(self._cv2.CAP_PROP_FRAME_COUNT))
#
#     def get_duration(self) -> int:
#         fps = self.get_fps()
#         frame_count = self.get_frame_count()
#         return math.ceil(frame_count / fps)



# -*- coding: utf-8 -*-
import datetime
from django.core.exceptions import ValidationError
from django.db.models.fields import DurationField as DjangoDurationField
from django.utils.dateparse import parse_duration

from a3dj.structs.base.base_field import BaseField, BaseFieldMockManager, BaseFieldOpenApiManager, JsonType, FormType
from a3dj.structs.mockers import DurationMocker, BaseMocker


class DurationFieldOpenApiManager(BaseFieldOpenApiManager):

    @classmethod
    def _preset_form_type(cls) -> str:
        return FormType.String

    @classmethod
    def _preset_open_api_format(cls) -> str:
        return 'duration'

    @classmethod
    def _preset_json_type(cls) -> str:
        return JsonType.String


class DurationFieldMockManager(BaseFieldMockManager):

    def _preset_default_mocker(self) -> BaseMocker:
        return DurationMocker()


class DurationField(BaseField):

    @classmethod
    def support_form(cls) -> bool:
        return True

    @classmethod
    def support_json(cls) -> bool:
        return True

    @classmethod
    def _build_from_model_field(cls, field_instance: DjangoDurationField) -> 'DurationField':
        return cls()

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return DurationFieldOpenApiManager(self)

    def _preset_mock_manager(self) -> BaseFieldMockManager:
        return DurationFieldMockManager(self)

    def to_python(self, value) -> datetime.timedelta:
        if isinstance(value, datetime.timedelta):
            return value

        try:
            parsed = parse_duration(value)
        except ValueError:
            pass
        else:
            if parsed is not None:
                return parsed

        raise ValidationError(
            DjangoDurationField.default_error_messages["invalid"],
            code="invalid",
            params={"value": value},
        )

    def to_json(self, value: datetime.timedelta) -> str:
        return str(value)



# -*- coding: utf-8 -*-
from typing import Optional

from .base_mocker import BaseMocker


class FixedValueMocker(BaseMocker):

    def __init__(self, fixed_value):
        super().__init__()
        self.fixed_value = fixed_value

    def mock(self, _, __):
        return self.fixed_value

    def to_json_mock_string(self) -> Optional[str]:
        return f"{self.fixed_value}"

    def to_form_mock_string(self) -> Optional[str]:
        return f"{self.fixed_value}"



# -*- coding: utf-8 -*-
import datetime
from django.core.exceptions import ValidationError
from django.db.models.fields import TimeField as DjangoTimeField

from a3dj.structs.base.base_field import BaseField, BaseFieldMockManager, BaseFieldOpenApiManager, JsonType, FormType
from a3dj.structs.mockers import TimeMocker, BaseMocker


class TimeFieldOpenApiManager(BaseFieldOpenApiManager):

    @classmethod
    def _preset_form_type(cls) -> str:
        return FormType.String

    @classmethod
    def _preset_open_api_format(cls) -> str:
        return 'time'

    @classmethod
    def _preset_json_type(cls) -> str:
        return JsonType.String


class TimeFieldMockManager(BaseFieldMockManager):

    def _preset_default_mocker(self) -> BaseMocker:
        return TimeMocker()


class TimeField(BaseField):

    @classmethod
    def support_form(cls) -> bool:
        return True

    @classmethod
    def support_json(cls) -> bool:
        return True

    @classmethod
    def _build_from_model_field(cls, field_instance: DjangoTimeField) -> 'TimeField':
        return cls()

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return TimeFieldOpenApiManager(self)

    def _preset_mock_manager(self) -> BaseFieldMockManager:
        return TimeFieldMockManager(self)

    def to_python(self, value) -> datetime.time:
        if isinstance(value, datetime.time):
            return value

        try:
            return datetime.time.fromisoformat(str(value)).replace(tzinfo=None)
        except ValueError:
            raise ValidationError(
                DjangoTimeField.default_error_messages["invalid_time"],
                code="invalid_time",
                params={"value": value},
            )

    def to_json(self, value: datetime.time) -> str:
        return str(value)



# -*- coding: utf-8 -*-
from django.core.exceptions import ValidationError
from django.db.models import FloatField as ModelFloatField

from a3dj.structs.base.base_field import BaseField, BaseFieldMockManager, BaseFieldOpenApiManager, JsonType, FormType
from a3dj.structs.mockers import FloatMocker, BaseMocker


class FloatFieldOpenApiManager(BaseFieldOpenApiManager):
    field: 'FloatField'

    @classmethod
    def _preset_form_type(cls) -> str:
        return FormType.Number

    @classmethod
    def _preset_open_api_format(cls) -> str:
        return 'float'

    @classmethod
    def _preset_json_type(cls) -> str:
        return JsonType.Number


class FloatFieldMockManager(BaseFieldMockManager):
    field: 'FloatField'

    def _preset_default_mocker(self) -> BaseMocker:
        return FloatMocker()


class FloatField(BaseField):

    @classmethod
    def support_form(cls) -> bool:
        return True

    @classmethod
    def support_json(cls) -> bool:
        return True

    @classmethod
    def _build_from_model_field(cls, field_instance: ModelFloatField) -> 'FloatField':
        return cls()

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return FloatFieldOpenApiManager(self)

    def _preset_mock_manager(self) -> BaseFieldMockManager:
        return FloatFieldMockManager(self)

    def to_python(self, value) -> float:
        if isinstance(value, float):
            return value

        try:
            return float(str(value))
        except (TypeError, ValueError):
            raise ValidationError(
                message=ModelFloatField.default_error_messages["invalid"],
                code="invalid",
                params={"value": value},
            )

    def to_json(self, value: float) -> float:
        return value



# -*- coding: utf-8 -*-
from .base_form_struct import BaseFormStruct
from .form_struct import FormStruct, FormStructOptions
from .model_form_struct import ModelFormStruct, ModelFormStructOptions
from ..fields import BooleanField, CharField, IntegerField, DateField, TimeField, DateTimeField, DurationField, \
    DecimalField, EmailField, FloatField, GenericIPAddressField, PositiveIntegerField, URLField, UUIDField, \
    JSONField, BinaryField, FileField, ImageField, ListField



# -*- coding: utf-8 -*-
import logging
from django.http import HttpRequest, Http404
from django.utils.deprecation import MiddlewareMixin

from a3dj.errors import Error, SUnknownError, SKnownError
from a3dj.core.request_utils import build_critical_message
from a3dj.core.response_utils import build_error_response
from a3dj.core.inner_errors import BaseInnerError


class ExceptionMiddleware(MiddlewareMixin):
    """
    通常放在最上面，这样可以处理所有的异常

    """

    def process_exception(self, request: HttpRequest, exception: Exception):
        logger = logging.getLogger(f'a3dj.{self.__class__.__name__}')

        if isinstance(exception, Error):
            error_response = build_error_response(exception)

            if exception.status.startswith('S'):
                logger.critical(build_critical_message(request, exception))
            else:
                error_message = f'[{request.path}]-[{exception.status}]: {str(exception)};;{repr(exception)}'
                logger.info(error_message)
        elif isinstance(exception, BaseInnerError):
            error_response = build_error_response(SKnownError())
            logger.critical(build_critical_message(request, exception))
        elif isinstance(exception, Http404):
            raise exception
        else:
            error_response = build_error_response(SUnknownError())
            logger.critical(build_critical_message(request, exception))

        return error_response



# -*- coding: utf-8 -*-
from a3dj.utils.hint_utils import TypeA3Model

from ..model_struct import ModelStructMetaClass, ModelStructMixin, ModelStructOptions
from .base_json_struct import BaseJsonStruct
from .json_struct import JsonStructMetaClass


ModelJsonStructOptions = ModelStructOptions


class ModelJsonStructMetaClass(ModelStructMetaClass):
    _check_field = JsonStructMetaClass._check_field # noqa


class ModelJsonStruct(BaseJsonStruct, ModelStructMixin, metaclass=ModelJsonStructMetaClass):
    _meta: ModelStructOptions

    def __init__(self, data=None, instance: TypeA3Model = None, **kwargs):
        BaseJsonStruct.__init__(self, data=data, **kwargs)
        ModelStructMixin.__init__(self, instance)

    def clean(self):
        ModelStructMixin.clean(self)



# -*- coding: utf-8 -*-
from django.core.exceptions import ValidationError
from django.db.models import Field as ModelField
from django.db.models.fields import BooleanField as DjangoBooleanField

from a3dj.structs.base.base_field import BaseField, BaseFieldMockManager, BaseFieldOpenApiManager, JsonType, FormType
from a3dj.structs.mockers import BooleanMocker, BaseMocker


class BooleanFieldOpenApiManager(BaseFieldOpenApiManager):

    @classmethod
    def _preset_form_type(cls) -> str:
        return FormType.String

    @classmethod
    def _preset_open_api_format(cls) -> str:
        return 'boolean'

    @classmethod
    def _preset_json_type(cls) -> str:
        return JsonType.Boolean


class BooleanFieldMockManager(BaseFieldMockManager):

    def _preset_default_mocker(self) -> BaseMocker:
        return BooleanMocker()


class BooleanField(BaseField):

    @classmethod
    def support_form(cls) -> bool:
        return True

    @classmethod
    def support_json(cls) -> bool:
        return True

    @classmethod
    def _build_from_model_field(cls, field_instance: ModelField) -> 'BooleanField':
        return cls()

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return BooleanFieldOpenApiManager(self)

    def _preset_mock_manager(self) -> BaseFieldMockManager:
        return BooleanFieldMockManager(self)

    def to_python(self, value) -> bool:
        if isinstance(value, bool):
            return value

        value = str(value).lower()

        if value in ('t', 'true', '1'):
            return True

        if value in ('f', 'false', '0'):
            return False

        raise ValidationError(
            message=DjangoBooleanField.default_error_messages["invalid"],
            code="invalid",
            params={"value": value},
        )

    def to_json(self, value: bool) -> bool:
        return value



# -*- coding: utf-8 -*-
from typing import Dict, Type
from django.db import models

from a3dj.structs.base.base_field import BaseField
from a3dj.structs import fields
from a3dj.core.inner_errors import ModelFieldTranslateError


class RelatedFieldTranslater:
    cached_mappings = dict()
    mappings: Dict[Type[models.Field], Type[BaseField]] = {
        models.IntegerField: fields.IntegerField,
        models.CharField: fields.CharField,
        models.BooleanField: fields.BooleanField
    }

    @classmethod
    def translate(cls, field_instance: models.Field) -> BaseField:
        json_field_cls = cls._get_struct_field_cls(type(field_instance))
        return json_field_cls.build_from_model_field(field_instance)

    @classmethod
    def _get_struct_field_cls(cls, field_cls) -> Type[BaseField]:
        cls_name = field_cls.__name__

        cache_result = cls.cached_mappings.get(cls_name, None)
        if cache_result is not None:
            return cache_result

        for f_cls in field_cls.__mro__:
            json_field_cls = cls.mappings.get(f_cls, None)
            if json_field_cls is not None:
                cls.cached_mappings[cls_name] = json_field_cls
                return json_field_cls

        raise ModelFieldTranslateError(f"{cls_name} can not find corresponding field class.")



# -*- coding: utf-8 -*-
import base64
import hashlib
from typing import Tuple, Any

from django.db.models import BinaryField as ModelBinaryField

from a3dj.core.utils import set_meaning_kv
from a3dj.core.validators import FileMaxSizeValidator
from a3dj.structs.base.base_field import BaseField, BaseFieldMockManager, BaseFieldOpenApiManager, JsonType, FormType
from a3dj.structs.mockers import NoneMocker, BaseMocker


class BinaryFieldOpenApiManger(BaseFieldOpenApiManager):
    field: 'BinaryField'

    @classmethod
    def _preset_form_type(cls) -> str:
        return FormType.String

    @classmethod
    def _preset_open_api_format(cls) -> str:
        return 'base64'

    @classmethod
    def _preset_json_type(cls) -> str:
        return JsonType.String

    def _build_json_base_info(self) -> dict:
        info = super()._build_json_base_info()
        set_meaning_kv(info, 'maxLength', self.field.max_length)

        return info


class BinaryFieldMockManager(BaseFieldMockManager):
    field: 'BinaryField'

    def _preset_default_mocker(self) -> BaseMocker:
        return NoneMocker()


class BinaryField(BaseField):

    @classmethod
    def support_form(cls) -> bool:
        return True

    @classmethod
    def support_json(cls) -> bool:
        return True

    @classmethod
    def _build_from_model_field(cls, field_instance: ModelBinaryField) -> 'BinaryField':
        return cls(max_length=field_instance.max_length)

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return BinaryFieldOpenApiManger(self)

    def _preset_mock_manager(self) -> BaseFieldMockManager:
        return BinaryFieldMockManager(self)

    def __init__(self, max_length: int = None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.max_length = max_length

        if self.max_length is not None:
            self.validators.append(FileMaxSizeValidator(self.max_length))

    def to_python(self, value) -> memoryview:
        if isinstance(value, memoryview):
            return value
        elif isinstance(value, (bytes, bytearray)):
            return memoryview(value)

        return memoryview(base64.b64decode(str(value).encode("ascii")))

    def to_json(self, value: memoryview) -> str:
        return base64.b64encode(value).decode("ascii")

    def check_has_changed(self, old_value: memoryview, new_value: memoryview) -> Tuple[bool, Any, Any]:
        def _get_md5(v) -> str:
            md5 = hashlib.md5()
            md5.update(v)
            return md5.hexdigest()

        if old_value is None:
            o = None
        else:
            o = _get_md5(old_value)

        if new_value is None:
            n = None
        else:
            n = _get_md5(new_value)
        return super().check_has_changed(o, n)



# -*- coding: utf-8 -*-
from typing import List, Tuple
from django.core.validators import MinLengthValidator, MaxLengthValidator, RegexValidator
from django.db.models import CharField as ModelCharField

from a3dj.structs.base.base_field import BaseField, BaseFieldMockManager, BaseFieldOpenApiManager, JsonType, FormType
from a3dj.structs.mockers import CharMocker, BaseMocker
from a3dj.core.utils import set_meaning_kv

from .utils.choices_utils import validate_choices, choices2open_api_dict

TypeCharChoices = List[Tuple[str, str]]


class CharFieldOpenApiManger(BaseFieldOpenApiManager):
    field: 'CharField'

    @classmethod
    def _preset_form_type(cls) -> str:
        return FormType.String

    @classmethod
    def _preset_open_api_format(cls) -> str:
        return 'string'

    @classmethod
    def _preset_json_type(cls) -> str:
        return JsonType.String

    def _build_json_base_info(self) -> dict:
        info = super()._build_json_base_info()
        set_meaning_kv(info, 'minLength', self.field.min_length)
        set_meaning_kv(info, 'maxLength', self.field.max_length)
        set_meaning_kv(info, 'pattern', self.field.pattern)
        if self.field.choices is not None:
            d = choices2open_api_dict(choices=self.field.choices)
            info.update(d)

        return info


class CharFieldMockManager(BaseFieldMockManager):
    field: 'CharField'

    def _preset_default_mocker(self) -> BaseMocker:
        return CharMocker(
            min_length=self.field.min_length, max_length=self.field.max_length, choices=self.field.choices
        )


class CharField(BaseField):

    @classmethod
    def support_form(cls) -> bool:
        return True

    @classmethod
    def support_json(cls) -> bool:
        return True

    @classmethod
    def _build_from_model_field(cls, field_instance: ModelCharField) -> 'CharField':
        return cls(max_length=field_instance.max_length, choices=field_instance.choices)

    def _preset_open_api_manager(self) -> BaseFieldOpenApiManager:
        return CharFieldOpenApiManger(self)

    def _preset_mock_manager(self) -> BaseFieldMockManager:
        return CharFieldMockManager(self)

    def __init__(
            self, min_length: int = None, max_length: int = None, choices: TypeCharChoices = None, pattern: str = None,
            *args, **kwargs
    ):
        super().__init__(*args, **kwargs)

        self.min_length = min_length
        self.max_length = max_length
        self.choices = choices
        self.pattern = pattern

        if self.min_length is not None:
            self.validators.append(MinLengthValidator(self.min_length))

        if self.max_length is not None:
            self.validators.append(MaxLengthValidator(self.max_length))

        if self.pattern is not None:
            self.validators.append(RegexValidator(self.pattern))
        else:
            for v in self.validators:
                if isinstance(v, RegexValidator):
                    self.pattern = str(v.regex.pattern)

    def validate(self, value, struct_instance):
        value = super().validate(value, struct_instance)

        if self.choices is not None and value is not None:
            value = validate_choices(choices=self.choices, value=value)

        return value

    def to_python(self, value) -> str:
        if isinstance(value, str):
            return value
        else:
            return str(value)

    def to_json(self, value: str) -> str:
        return value



# -*- coding: utf-8 -*-
from django.utils.translation import gettext_lazy as _

from .error import Error


class ClientError(Error):
    default_message = _('Client error.')


class CRequestStructError(ClientError):
    default_message = _('The requested data structure is not in the correct format.')



# -*- coding: utf-8 -*-
from .base_mocker import BaseMocker


class ObjectMocker(BaseMocker):

    def __init__(self, obj_cls):
        super().__init__()
        self.obj_cls = obj_cls

    def mock(self, instance, root_instance):
        return self.obj_cls.mock(instance=instance, root_instance=root_instance)

    def to_json_mock_string(self):
        return

    def to_form_mock_string(self):
        return



# -*- coding: utf-8 -*-
from typing import Type

from a3dj.structs.base.struct import Struct


class BaseJsonStruct(Struct):

    @classmethod
    def to_application_open_api_dict(cls) -> dict:
        return {
            'application/json': {
                "schema": cls.to_json_open_api_dict()
            }
        }

    @classmethod
    def to_json_open_api_dict(cls) -> dict:
        if cls.is_open_api_component():
            rd = {
                "$ref": f"#/components/schemas/{cls.__name__}"
            }
        else:
            rd = cls.to_open_api_component_dict()

        return rd

    @classmethod
    def is_open_api_component(cls) -> bool:
        return cls._meta.is_component

    @classmethod
    def to_open_api_component_dict(cls) -> dict:
        order_list = list()
        required_list = list()
        properties = dict()

        for field_name, field_instance in cls._meta.fields.items():
            order_list.append(field_name)

            if field_instance.required:
                required_list.append(field_name)

            properties[field_name] = field_instance.open_api_manager.to_json_open_api_dict()

        return {
            'type': 'object',
            'properties': properties,
            'required': required_list,
            'x-apifox-orders': order_list
        }

    @classmethod
    def get_open_api_components(cls) -> dict[str, Type['BaseJsonStruct']]:
        components = dict()
        if cls.is_open_api_component():
            components[cls.__name__] = cls
        else:
            for _, field_instance in cls.get_meta().fields.items():
                sub_components = field_instance.open_api_manager.get_components()
                if isinstance(sub_components, dict) and len(sub_components) > 0:
                    components.update(sub_components)

        return components
